# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico

import cwidgets

def gen_group(title, chk, help=None):

    widget = QtWidgets.QGroupBox()
    widget.setTitle(title)
    widget.setCheckable(chk)
    lay = QtWidgets.QGridLayout()
    widget.setLayout(lay)

    if help:
        lbl = cwidgets.XLabel()
        lbl.setText(help)
        lbl.setStyleSheet("background-color: white; color: #000099; padding: 4px; border-radius: 4px;")
        #lbl.setMaximumWidth(300)
        lbl.setWordWrap(True)
        lay.addWidget(lbl, 0, 0, 1, 2)
    lay.setColumnStretch(0, 0)
    lay.setColumnStretch(1, 1)
    return widget


class SettingsDialog(QtWidgets.QDialog):

    sigSaved = pyqtSignal(dict)
    sigDeleted = pyqtSignal(str)

    @property
    def url(self):
        return "/settings"

    def __init__(self, parent=None, serverConn=None):
        super().__init__(parent)

        assert serverConn != None

        self.serverConn = serverConn

        self.setWindowTitle("Settings")
        self.setWindowIcon(Ico.icon(Ico.settings))
        self.setMinimumWidth(500)
        self.setMaximumWidth(600)


        self.mainLayout = cwidgets.vlayout(margin=20)
        self.setLayout(self.mainLayout)


        self.grid = QtWidgets.QGridLayout()
        self.grid.setSpacing(10)
        self.mainLayout.addLayout(self.grid)

        row = 0
        hlp = "Modbus slave enables reading and writing located variables through the" \
            "modbus interface. This starts a Modbus server (also know as a slave) " \
            "running in the OpenPLC runtime."
        self.grpModbusSlave = gen_group("Modbus Slave Enabled", True, help=hlp)
        self.grid.addWidget(self.grpModbusSlave, row, 0)

        self.grpModbusSlave.layout().addWidget(QtWidgets.QLabel("Port"), 1, 0, Qt.AlignRight)
        self.spinModbusSlavePort = QtWidgets.QSpinBox()
        self.spinModbusSlavePort.setRange(0, 30000)
        self.spinModbusSlavePort.setMaximumWidth(80)
        self.grpModbusSlave.layout().addWidget(self.spinModbusSlavePort, 1, 1)

        row += 1
        hlp = "Modbus master enables reading and writing located variables through the " \
                "modbus interface. This starts capabilities to poll one or more Modbus " \
                " servers and exchange data with the located variables."
        self.grpModbusMaster = gen_group("Modbus Master Enabled", True, help=hlp)
        self.grid.addWidget(self.grpModbusMaster, row, 0)

        self.grpModbusMaster.layout().addWidget(QtWidgets.QLabel("Address"), 1, 0, Qt.AlignRight)
        self.txtModbusMasterAddress = cwidgets.XLineEdit()
        self.grpModbusMaster.layout().addWidget(self.txtModbusMasterAddress, 1, 1)

        self.grpModbusMaster.layout().addWidget(QtWidgets.QLabel("Port"), 2, 0, Qt.AlignRight)
        self.spinModbusMasterPort = QtWidgets.QSpinBox()
        self.spinModbusMasterPort.setRange(0, 30000)
        self.spinModbusMasterPort.setMaximumWidth(80)
        self.grpModbusMaster.layout().addWidget(self.spinModbusMasterPort, 3, 1)


        row += 1
        self.grpDnp = gen_group("DNP3 Enabled", True)
        self.grid.addWidget(self.grpDnp, row, 0)

        self.grpDnp.layout().addWidget(QtWidgets.QLabel("Port"), 0, 0, Qt.AlignRight)
        self.spinDNP3Port = QtWidgets.QSpinBox()
        self.spinDNP3Port.setRange(0, 30000)
        self.grpDnp.layout().addWidget(self.spinDNP3Port, 0, 1)

        row += 1
        self.grpSlave = gen_group("Slave", False)
        self.grid.addWidget(self.grpSlave, row, 0)

        self.grpSlave.layout().addWidget(QtWidgets.QLabel("Polling (ms)"), 0, 0, Qt.AlignRight)
        self.spinSlavePolling = QtWidgets.QSpinBox()
        self.spinSlavePolling.setRange(0, 30000)
        self.grpSlave.layout().addWidget(self.spinSlavePolling, 0, 1)

        self.grpSlave.layout().addWidget(QtWidgets.QLabel("Timeout (ms)"), 1, 0, Qt.AlignRight)
        self.spinSlaveTimeout = QtWidgets.QSpinBox()
        self.spinSlaveTimeout.setRange(0, 30000)
        self.grpSlave.layout().addWidget(self.spinSlaveTimeout, 1, 1)

        self.grid.setColumnStretch(0, 2)
        #self.grid.setColumnStretch(1, 1)
        #self.grid.setColumnStretch(1, 1)
        #self.grid.setColumnStretch(2, 2)

        self.mainLayout.addSpacing(20)

        self.formBar = cwidgets.FormActionBar(self, show_meta=False)
        self.mainLayout.addWidget(self.formBar)

        self.init_load()

    def init_load(self):
        self.serverConn.get(self, self.url)

    def on_server_reply(self, reply):

        if reply.busy:
            return


        if reply.post:

            if reply.data.get("saved") == True:
                self.sigSaved.emit(reply.data['settings'])
                self.accept()
            return

        self.formBar.set_reply(reply)

        rec = reply.data.get('settings')

        #self.grpModbus.setChecked(rec["modbus_enabled"])
        #self.spinModbusPort.setValue(rec['modbus_port'])

        self.grpDnp.setChecked(rec["dnp3_enabled"])
        self.spinDNP3Port.setValue(rec['dnp3_port'])

        self.spinSlavePolling.setValue(rec['slave_polling'])
        self.spinSlaveTimeout.setValue(rec['slave_timeout'])



        self.formBar.set_meta("prog_id", rec)

        self.setDisabled(False)




    def on_cancel(self):
        self.reject()

    def on_save(self):


        rec = dict(modbus_enabled = self.grpModbus.isChecked(),
                   modbus_port = self.spinModbusPort.value(),
                   dnp3_enabled = self.grpDnp.isChecked(),
                   dnp3_port = self.spinDNP3Port.value(),
                   slave_polling = self.spinSlavePolling.value(),
                   slave_timeout = self.spinSlaveTimeout.value(),
                   )

        self.serverConn.post(self, self.url, data=rec)


    def on_delete(self):
        self.sigDeleted.emit(self.user_id)
        self.accept()

    def on_set_pass(self, state=False):
        self.txtPass.setEnabled(self.chkPass.isChecked())


# -*- coding: utf-8 -*-

import os
import sys


from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import G
import cwidgets
from img import Ico

from server import server_dialog, server_panel, servers_dialog

class MainWindow(QtWidgets.QWidget):

    def on_after(self):


        if os.path.exists("PEDRO.txt"):
            pass #QtCore.QTimer.singleShot(300, self.maximizeWindow)


        for srv in G.settings.servers_list():
            if srv['auto_connect']:
                self.open_server(srv)

    def __init__(self, parent=None, args=None):
        QtWidgets.QWidget.__init__(self, parent)

        G.args = args

        self.setWindowTitle("OpenPLC Desktop")
        self.setWindowIcon(Ico.logo())

        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        # menu
        self.menuBar = QtWidgets.QMenuBar()
        self.mainLayout.addWidget(self.menuBar)

        self.menuFile = QtWidgets.QMenu("File")
        self.menuBar.addMenu(self.menuFile)

        self.menuFile.addAction(Ico.icon(Ico.quit), "Quit", self.on_quit)


        self.menuServers = QtWidgets.QMenu("Servers")
        self.menuBar.addMenu(self.menuServers)

        self.menuServers.addAction(Ico.icon(Ico.servers), "Servers", self.on_servers)

        self.menuServers.addSeparator()
        self.menuServers.addAction(Ico.icon(Ico.add), "Add Server", self.on_add_server)


        ## Top toolbar
        self.topToolbar = cwidgets.hlayout()
        self.mainLayout.addLayout(self.topToolbar, 0)

        self.lblHeader = QtWidgets.QLabel()
        self.topToolbar.addWidget(self.lblHeader, 10)
        self.lblHeader.setText("OpenPLC")
        style_grad = "background-color: qlineargradient(x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #444444, stop: 1 #D95343);"
        style_grad += "font-size: 18pt; color: #efefef; padding: 5px 10px;"
        self.lblHeader.setStyleSheet(style_grad)

        self.buttAddServer = cwidgets.XToolButton(self, text="Add Server", ico=Ico.add, callback=self.on_add_server)
        self.topToolbar.addWidget(self.buttAddServer)

        if 1:
            self.devToolbar = QtWidgets.QToolBar()
            self.mainLayout.addWidget(self.devToolbar)

            self.buttNoNo = QtWidgets.QPushButton()
            self.buttNoNo.setText("Magic Button")
            self.buttNoNo.setToolTip("Though shalt not click this button before 6am gmt")
            self.devToolbar.addWidget(self.buttNoNo)
            self.buttNoNo.clicked.connect(self.on_nono_clicked)



        tlay = cwidgets.hlayout()
        self.mainLayout.addLayout(tlay)
        self.tabBar = QtWidgets.QTabBar()
        tlay.addWidget(self.tabBar, 0)
        tlay.addStretch(5)

        self.appStack = QtWidgets.QStackedWidget()
        self.mainLayout.addWidget(self.appStack, 20)

        if 0:
            self.serverPanel = server_panel.ServerPanel()
            self.add_widget(self.serverPanel, "Server", Ico.server)

        if G.args.pedro:
            self.move(1800, 20)
            self.setMinimumWidth(600)
            self.setMinimumHeight(600)
            self.setWindowState(Qt.WindowMaximized)

        self.tabBar.currentChanged.connect(self.on_tab_changed)

        ## dev mode so switch to wizz
        # self.tabBar.setCurrentIndex(1)

        QtCore.QTimer.singleShot(200, self.on_after)

    def add_widget(self, widget, text, ico=None):
        self.appStack.addWidget(widget)
        idx = self.tabBar.addTab(text)
        if ico:
            self.tabBar.setTabIcon(idx, Ico.icon(ico))

        widget.init_load()

    def on_tab_changed(self, nidx):
        self.appStack.setCurrentIndex(nidx)

    def on_nono_clicked(self):
        print("panic")
        self.appStack.widget(0).load_data()

    def on_bank_holiday_clicked(self):
        print("on_bank_holiday", self)

    def on_add_server(self, autologin=False):

        dial = server_dialog.ServerDialog(self)
        dial.sigSaved.connect(self.on_server_saved)
        dial.exec_()

    def on_server_saved(self, server):

        if server:
            s = "%s\n%s" % (user['username'], user['email'])
            self.lblLogin.setText(s)

    def on_quit(self):
        sys.exit(0)

    def on_servers(self):

        dial = servers_dialog.ServersDialog(self)
        dial.exec_()

    def open_server(self, srv):

        widget = server_panel.ServerPanel(server=srv)
        self.add_widget(widget, srv['name'],ico=Ico.server)







import sys

# Import from here...
# another fork to fix things,, https://pyqtx.gitlab.io/pyqtermwidget/

sys.path.append("/home/oplc/pyqtermwidget/")


# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, QtNetwork, Qt, pyqtSignal

from img import Ico

import cwidgets

import pyqterm


class PyQTermWidget(QtWidgets.QWidget):

    def __init__(self, parent, serverConn=None):
        QtWidgets.QWidget.__init__(self, parent)

        assert serverConn != None
        self.serverConn = serverConn





        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        self.toolbar = cwidgets.XToolBar()
        self.mainLayout.addWidget(self.toolbar)

        ## ------------ credentials
        grp = cwidgets.ToolBarGroup(title="Terminal")
        self.toolbar.addWidget(grp)

        self.txtHost = cwidgets.XLineEdit()
        grp.addWidget(self.txtHost)
        self.txtHost.setText("192.168.1.222")
        self.txtHost.setMaximumWidth(120)

        self.spinPort = QtWidgets.QSpinBox()
        grp.addWidget(self.spinPort)
        self.spinPort.setValue(22)

        self.txtUser = cwidgets.XLineEdit()
        grp.addWidget(self.txtUser)
        self.txtUser.setText("pi")
        self.txtUser.setMaximumWidth(80)

        self.txtPass = cwidgets.XLineEdit()
        grp.addWidget(self.txtPass)
        self.txtPass.setText("raspberry")
        self.txtPass.setMaximumWidth(80)

        #self.buttConnect = cwidgets.XToolButton(ico=Ico.start, text="Connect", callback=self.on_connect)
        #grp.addWidget(self.buttConnect)

        self.lblStatus = cwidgets.XLabel()
        grp.addWidget(self.lblStatus)



        ## ------------ send command
        grp = cwidgets.ToolBarGroup(title="Execute")
        self.toolbar.addWidget(grp)

        self.txtCommand = cwidgets.XLineEdit()
        grp.addWidget(self.txtCommand)
        self.txtCommand.setText("ls -l")
        #self.txtCommand.returnPressed.connect(self.on_execute)

        #self.buttConnect = cwidgets.XToolButton(ico=Ico.return_key, text="Execute", callback=self.on_execute)
        #grp.addWidget(self.buttConnect)



        self.terminal = pyqterm.TerminalWidget(self)
        self.mainLayout.addWidget(self.terminal)



    def init_load(self):
        pass





from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico
import G
import cwidgets

from server import server_conn

class ServerDialog(QtWidgets.QDialog):

    sigSaved = pyqtSignal(dict)


    def __init__(self, parent=None, server=None):
        super().__init__(parent)



        self.setWindowTitle("Server Details")
        self.setWindowIcon(Ico.icon(Ico.server))
        self.setMinimumWidth(400)



        self.mainLayout = cwidgets.vlayout(margin=20)
        self.setLayout(self.mainLayout)


        self.grid = QtWidgets.QGridLayout()
        self.grid.setSpacing(10)
        self.mainLayout.addLayout(self.grid)

        row = 0
        self.grid.addWidget(QtWidgets.QLabel("Label / Name"), row, 0, Qt.AlignRight)
        self.txtServerName = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtServerName, row, 1)
        self.txtServerName.setPlaceholderText("eg My Server")

        row  += 1
        self.grid.addWidget(QtWidgets.QLabel("Server Address"), row, 0, Qt.AlignRight)
        self.txtServerAddress = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtServerAddress, row, 1, 1, 2)
        self.txtServerAddress.textChanged.connect(self.on_validate)

        self.buttPing = cwidgets.XToolButton(self, text="Ping", autoRaise=False, callback=self.on_ping)
        self.grid.addWidget(self.buttPing, row, 3)

        row += 1
        self.grid.addWidget(QtWidgets.QLabel("User"), row, 0, Qt.AlignRight)
        self.txtUser = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtUser, row, 1)

        row += 1
        self.grid.addWidget(QtWidgets.QLabel("Password"), row, 0, Qt.AlignRight)
        self.txtPass = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtPass, row, 1)

        row += 1

        self.buttTestLogin = cwidgets.XToolButton(self, text="Test Login", autoRaise=False, callback=self.on_test_login)
        self.grid.addWidget(self.buttTestLogin, row, 1, Qt.AlignRight)


        row += 1
        self.chkAutoConnect = QtWidgets.QCheckBox()
        self.chkAutoConnect.setText("Auto connect at startup")
        self.grid.addWidget(self.chkAutoConnect, row, 1)

        row += 1
        self.txtMess = QtWidgets.QLabel("")
        self.grid.addWidget(self.txtMess, row, 1)

        self.statusBar = cwidgets.StatusBar(self, refresh=False)
        self.mainLayout.addWidget(self.statusBar)

        self.formBar = cwidgets.FormActionBar(self, show_meta=False)
        self.mainLayout.addWidget(self.formBar)

        self.formBar.buttSave.setIcon(Ico.icon(Ico.login))
        self.formBar.buttSave.setText("Login")

        if server:
            self.load(server)

    def load(self, srv):
        self.txtServerName.setText(srv['name'])
        self.txtServerAddress.setText(srv['address'])
        self.txtUser.setText(srv['login'])
        self.txtPass.setText(srv['password'])
        self.chkAutoConnect.setChecked(srv['auto_connect'])


    def on_cancel(self):
        self.reject()

    def on_validate(self):

        s = self.txtServerAddress.s()
        url = QtCore.QUrl(s)
        ena = False
        if url.isValid() and url.scheme() in ["http", "https"]:
            ena = True
        self.buttPing.setEnabled(ena)


    def on_save(self):

        for w in self.txtServerName, self.txtServerAddress, self.txtUser, self.txtPass:
            if len(w.s()) < 4:
                #w.borderInvalid()
                w.setFocus()
                return

        srv = dict(
            name = self.txtServerName.s(),
            address = self.txtServerAddress.s(),
            login = self.txtUser.s(),
            password = self.txtPass.s(),
            auto_connect = self.chkAutoConnect.isChecked()
        )
        G.settings.save_server(srv)


        self.accept()




    def on_ping(self):
        addr = self.txtServerAddress.s()
        server = server_conn.ServerConn(self, server_address=addr)
        server.get(self, "/ping", tag="ping")

    def on_test_login(self):

        data = dict(login=self.txtUser.s(), password=self.txtPass.s())

        addr = self.txtServerAddress.s()
        server = server_conn.ServerConn(self, server_address=addr)
        server.post(self, "/login", tag="login", data=data)

    def on_server_reply(self, reply):

        self.statusBar.set_reply(reply)
        if reply.busy:
            return

        if reply.error:
            self.statusBar.set_reply(reply)
            return


        if reply.tag == "ping":
            if reply.data['ping'] == "pong":
                self.statusBar.showMessage("PING Success :-)", info=True, timeout=3000)

        if reply.tag == "login":
            if reply.data['logged_in']:
                self.statusBar.showMessage("Login Success :-)", info=True, timeout=3000)
            else:
                self.statusBar.showMessage("Login fail", timeout=3000)




# -*- coding: utf-8 -*-

import os
import urllib
import json

from Qt import Qt, QtCore




class XSettings( QtCore.QObject ):

    CURR_SERVER_KI = "current/server_ki"

    def __init__( self, parent=None ):
        QtCore.QObject.__init__( self, parent )

        self.qsettings = QtCore.QSettings("OpenPLC", "openplc-desktop-pyqt")


        self.server_auth = False


    def servers_list(self):
        v = self.qsettings.value("servers")
        if not v:
            return []
        return list(v.values())

    def save_server(self, srv):
        servs = self.qsettings.value("servers")
        if not servs:
            servs = {}
        servs[srv['name']] = srv
        self.qsettings.setValue("servers", servs)
        self.qsettings.sync()

    def remove_server(self, srv):
        servs = self.qsettings.value("servers")
        if not servs:
            servs = {}
        if srv['name'] in servs:
            del servs[srv['name']]
        self.qsettings.setValue("servers", servs)
        self.qsettings.sync()

    ##==============================
    ## Window Save/Restore
    def save_window( self, window ):
        name = window.objectName()
        if len(name) == 0:
            return
        self.qsettings.setValue( "window/%s/geometry" % name, QtCore.QVariant( window.saveGeometry() ) )

    def restore_window( self, window ):
        name = str(window.objectName())
        if len(name) == 0:
            return
        window.restoreGeometry( self.qsettings.value( "window/%s/geometry" % name ) )


    def save_splitter(self, splitter):
        wname = str(splitter.objectName())
        if not wname:
            print ("Splitter has no name", splitter)
        self.qsettings.setValue("splitter/%s" % wname, splitter.saveState())

    def restore_splitter(self, splitter):
        wname = str(splitter.objectName())
        if not wname :
            print ("Splitter has no name")
            return
        v = self.qsettings.value("splitter/%s" % wname)
        if v:
            splitter.restoreState(v)

    def save_list(self, key, lst):
        self.qsettings.setValue(key, json.dumps(lst))
        self.qsettings.sync()

    def get_list(self, key):
        s = self.qsettings.value(key)
        if s == None or s == "":
            return []
        return json.loads(s)





# -*- coding: utf-8 -*-
# @author: Peter Morgan <pete@daffodil.uk.com>


import json
import mimetypes
import datetime

import paramiko
import websocket
from Qt import  Qt, QtCore, QtNetwork, QtWidgets, pyqtSignal


import G
from img import Ico


TIMEOUT = 5000 #30000
ATTR_TAG = QtNetwork.QNetworkRequest.User + 10
DEBUG_TAG = QtNetwork.QNetworkRequest.User + 20


class Reply:

    def __init__(self):

        self.debug = False
        self.busy = False
        self.url = None
        self.operation = None

        self.http_code = None

        self.error = None
        self.content = None
        self.data = None
        self.tag = None

    @property
    def get(self):
        return self.operation == QtNetwork.QNetworkAccessManager.GetOperation

    @property
    def post(self):
        return self.operation == QtNetwork.QNetworkAccessManager.PostOperation

    @property
    def status(self):
        return "BUSY" if self.busy else "IDLE"

    @property
    def method(self):
        if self.get:
            return "GET"
        if self.post:
            return "POST"
        return "?method?"

    def callback(self, s=" ??"):
        #print(" << callback " ,s,  self.origin)
        if self.origin == None:
            # TODO make cancel
            print("####### origin disappeared. .CANCEL ME TODO...", self)
            return
        self.origin.on_server_reply(self)

    def print(self):
        if self.busy:
            print(">>------------------------->>")
        else:
            print("<<-------------------------<<")
        print("%s %s %s" % (self.status, self.method, self.url))
        if self.tag:
            print("tag=", self.tag)
        if self.data == None:
            print(" ## NO DATA ##")
            return

        if self.debug:
            print(self.data)
        else:
            print(", ".join( sorted(self.data.keys()) ))


    def __repr__(self):
        d = None
        if self.data:
            if isinstance(self.data, dict):
                d = sorted(self.data.keys())
            elif isinstance(self.data, list):
                d = self.data
            else:
                d = self.data.kkkcrash()

        return "<Reply %s %s %s:%s, ori=%s, err=%s \n  data=%s>" % (
            self.status, self.http_code, self.method, self.url, self.origin.__class__.__name__,  self.error, d)

#
# class WebSockMess:
#     def __init__(self, rec):
#         self.ns = rec['ns']
#         self.mtype = rec['mtype']
#         self.data = rec['data']
#         self.ts = datetime.datetime.fromtimestamp(rec['ts'])
#


class WebsocketClient(QtCore.QThread):

    sigWsMessage = pyqtSignal(object)

    def __init__(self, parent=None):
        super(WebsocketClient, self).__init__(parent)

        self.debug = True
        websocket.enableTrace(True)

        self.WS = websocket.WebSocketApp("ws://localhost:8080/ws",
                                    on_message = self.on_message,
                                    on_error = self.on_error,
                                    on_ping=self.on_ping,
                                    on_pong=self.on_pong,
                                    on_open = self.on_open,
                                    on_close = self.on_close)

    def on_message(self, message):#
        recs = json.loads(message)
        self.sigWsMessage.emit(recs)

        if self.debug:
            print("on_message", message)

    def on_error(self, *args):
        if self.debug:
            print("error", args)

    def on_close(self, *args):
        if self.debug:
            print("### closed ###", args)


    def on_pong(self, ws):
        #if self.debug:
        #    print("on_pong", ws)
        pass

    def on_ping(self, ws):
        #if self.debug:
        #    print("on_ping", ws)
        pass



    def on_open(self):
        if self.debug:
            print("on_open")

    def on_close(self):
        if self.debug:
            print("on_close")

    def run(self):
        self.WS.run_forever(ping_interval=3)


class RuntimeMessage:

    def __init__(self, message=None, error=None, connected=None):

        self.message = message
        self.error = error
        self.connected = connected

class ServerConn( QtCore.QObject ):
    """
    HTTP Client for ajax queries
    """
    TIMEOUT = 3000 #30000

    sigWsMessage = pyqtSignal(object)
    sigRuntimeMessage = pyqtSignal(object)
    sigSSHMessage = pyqtSignal(object, object)

    def __init__( self, parent, server_address=None ):
        super(ServerConn, self).__init__(parent)

        self.netManager = QtNetwork.QNetworkAccessManager( self )

        self.server_address = server_address

        ## The cookie jar only lasts for session
        # with pyqt we are using settings to persists (ATMO)
        self.cookieJar = QtNetwork.QNetworkCookieJar()
        self.netManager.setCookieJar( self.cookieJar )

        self.netManager.finished.connect(self.on_REQUEST_FINISHED)

        self.progressDialog = None


        ## Websocket
        self.webSock = WebsocketClient(self)
        self.webSock.sigWsMessage.connect(self.on_ws_message)


        ## Runtime
        self.runtimeSock = QtNetwork.QTcpSocket(self)
        self.runtimeSock.connected.connect(self.on_runtime_connected)
        self.runtimeSock.disconnected.connect(self.on_runtime_disconnected)
        self.runtimeSock.readyRead.connect(self.on_runtime_socket_ready_read)
        self.runtimeSock.error.connect(self.on_runtime_socket_error)

        self.sshClient = paramiko.SSHClient()
        self.sshClient.load_system_host_keys()
        self.sshClient.set_missing_host_key_policy(paramiko.WarningPolicy)

    def connect_ssh(self, host, port, username, password):
        #print("on_ssh")
        self.sshClient.connect(host, port=port, username=username, password=password)
        self.send_ssh("uname -a")
        self.send_ssh("whoami")


    def send_ssh(self, cmd):
        stdin, stdout, stderr = self.sshClient.exec_command(cmd)
        # print( stdout.read(), )
        sout = str(stdout.read(), encoding="ascii")
        serr = str(stderr.read(), encoding="ascii")
        #print(s)
        self.sigSSHMessage.emit(sout, serr)

    def send_runtime(self, cmd):
        cmdd = cmd + "\n"
        #print("send_runtime", cmdd)
        bites = QtCore.QByteArray()
        bites.append(cmdd)
        self.runtimeSock.write(bites)

    def connect_runtime(self):
        url = QtCore.QUrl(self.server_address)
        #print("connect_runtime", url.host())
        self.runtimeSock.connectToHost(url.host(), 43628)

    def on_runtime_connected(self):
        #print("on_runtime_connected")
        m = RuntimeMessage(connected=True)
        self.sigRuntimeMessage.emit(m)
        self.send_runtime("runtime_logs()")

    def on_runtime_disconnected(self):
        #print("on_runtime_disconnected")
        m = RuntimeMessage(connected=False)
        self.sigRuntimeMessage.emit(m)

    def on_runtime_socket_ready_read(self):
        #print("on_runtime_socket_ready_read")
        bites = self.runtimeSock.readAll()
        s = str(bites, encoding="ascii")
        m = RuntimeMessage(connected=True, message=s)
        self.sigRuntimeMessage.emit(m)

    def on_runtime_socket_error(self, err):
        #print("on_runtime_socket_error", err)
        m = RuntimeMessage(connected=False, error=err)
        self.sigRuntimeMessage.emit(m)


    ## ======================================
    def connect_websocket(self):
        print("connext")

        self.webSock.start()
        return
        addr = "ws://localhost:8080/ws" # % self.server_address

    def on_ws_message(self, mess):
        self.sigWsMessage.emit(mess)



    @property
    def base_url(self):
        if self.server_address == None:
            return ""
        return self.server_address + "/ajax"



    def _make_request_obj(self, origin, url_str, tag=None, debug=None):

        qurl = QtCore.QUrl("%s%s" % (self.base_url, url_str))

        request = QtNetwork.QNetworkRequest()
        request.setOriginatingObject(origin)
        request.setPriority(QtNetwork.QNetworkRequest.HighPriority)
        if tag != None:
            request.setAttribute(ATTR_TAG, tag)
        if debug != None:
            request.setAttribute(DEBUG_TAG, debug)

        #self.set_raw_header(request, "Host", qurl.host)
        self.set_raw_header(request, "X-OPENPLC-CLIENT", "openplc-desktop")
        self.set_raw_header(request, "X-OPENPLC-VERSION", "0.0.1")

        return request, qurl, None


    #==================================================================
    ## GET to server
    def get( self, origin, url=None, params=None,  tag=None, debug=False):
             #cb=None, widget=None,  debug=False, spinner=True):

        request, qurl, err = self._make_request_obj(origin, url, tag=tag)
        if err:
            return err

        q = QtCore.QUrlQuery()
        if params:
            for k, v in params.items():
                q.addQueryItem( str( k ), str( v ) )
        qurl.setQuery(q)

        request.setUrl(qurl)

        reply = Reply()
        reply.debug = debug
        reply.busy = True


        reply.url = request.url().toString()
        reply.origin = origin
        reply.operation = QtNetwork.QNetworkAccessManager.GetOperation
        reply.callback("get()")

        if debug:
            reply.print()

        self.netManager.get( request )




    #==================================================================
    ## POST to the server
    def post( self, origin, url, data=None, tag=None, form=False, debug=False ):

        request, qurl, err = self._make_request_obj(origin, url, tag=tag)

        bites = QtCore.QByteArray()
        if form:
            postData = QtCore.QUrlQuery()
            for k in data:
                postData.addQueryItem( k, str( data[k] ) )

            bites.append(postData.toString(QtCore.QUrl.FullyEncoded))


        else:

            data_str = json.dumps(data)
            bites.append(data_str)
            self.set_raw_header(request, "Content-Type", "application/json")
            self.set_raw_header(request, "Content-Length", str(len(data_str)))

        request.setUrl(qurl)

        reply = Reply()
        reply.busy = True
        reply.debug = debug
        reply.origin = origin
        reply.operation = QtNetwork.QNetworkAccessManager.PostOperation
        reply.callback("post()")

        self.progress_start(url)
        if debug:
            reply.print()

        self.netManager.post( request, bites)


    def progress_start(self, url=None):
        self.progressDialog = QtWidgets.QProgressDialog()
        self.progressDialog.setMinimumWidth(300)
        self.progressDialog.setWindowTitle("POST: server")
        if url:
            self.progressDialog.setLabelText(url)
        #self.progressDialog.setWindowIcon(Ico.icon(Ico.busy))
        self.progressDialog.setRange(0, 0)
        #self.progressDialog.setCancelButton(True)
        #progressDialog.setModal(True)
        self.progressDialog.setWindowModality(Qt.WindowModal)
        self.progressDialog.forceShow()
        return
        #progressDialog.hide()
        #if origin:
         #   self.progressDialog.setParent(origin)
        self.progressDialog.setRange(0, 0)
        self.progressDialog.forceShow()

    def progress_stop(self):
        #print(">> progress_stop")
        if self.progressDialog == None:
            return
        self.progressDialog.setRange(0, 100)
        self.progressDialog.hide()
        self.progressDialog = None

    @staticmethod
    def set_raw_header(request, ki, vi):

        ha = QtCore.QByteArray()
        ha.append(ki)

        ba = QtCore.QByteArray()
        ba.append(vi)

        request.setRawHeader(ha, ba)





    #==========================================================
    ## Server Request Finished
    def on_REQUEST_FINISHED( self, qreply ):

        """Server Request has finished, so parse and check for errors"""
        #print("< fin", qreply.errorString(), qreply)
        reply = Reply()
        reply.busy = False

        reply.url = qreply.request().url().toString()
        reply.origin = qreply.request().originatingObject()
        #reply.origin.on_server_status(Status.idle)


        reply.http_code = qreply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
        reply.operation = qreply.operation()

        reply.tag = qreply.request().attribute(ATTR_TAG)
        reply.debug = qreply.request().attribute(DEBUG_TAG)




        turl = qreply.request().url()
        reply.url = turl.path() + "?" + turl.query()


        self.progress_stop()

        if qreply.error():
            reply.error = "%s: %s" % (qreply.error(), qreply.errorString())
            reply.callback("srv.err")
            qreply.deleteLater()
            return

        if reply.http_code != 200: # Not ok!
            print("GOT ERROR", reply.http_code)
            if reply.http_code == 301:
                reply.error = "Permissions error"
            else:
                reply.error = "ERROR: %s" % reply.http_code
            reply.callback("srv.err")
            qreply.deleteLater()
            return



        ##==============================
        ## Decode and handle json reply
        try:
            contents = str(qreply.readAll().data(), encoding="ascii")
            #print("contents=", contents)
            reply.data = json.loads(contents)
            #print("reply.data=", type(reply.data))
            #print("> data", xdate.now())



            # Always expect a dict so throw tantrum otherwise
            if isinstance(reply.data, dict) or isinstance(reply.data, list):


                if "error" in reply.data and reply.data["error"]:
                    reply.error = reply.data["error"]
                reply.callback("data.json")
                qreply.deleteLater()
                if reply.debug:
                    reply.print()
                return


            print("GOT=========", reply.data.keys())
            panic()




        ## Probably some server error type string - didnt happen in JSON
        except ValueError as e:

            reply.error = str(e)
            reply.callback("error.json")
            qreply.deleteLater()
            return

        ########################################
        ## Json Decoding was not correct ?
        except TypeError as e:

            reply.error = str(e)
            reply.callback("error.json")
            qreply.deleteLater()
            return
                #self.xStatusBar.serverButton.setTex
                #self.xStatusBar.serverButton = s
                #self.xStatusBar.serverString = self.srv_str

        ########################################################
        ## json is decoded and works
        else:
            ## json says No Success will always be true
            if 'error' in resp.data and self.json['error']:
                print("App Error")
                self.update_status( SERVER_STATUS.APP_ERROR )


            dddd()
        qreply.deleteLater()





    ##########################################################################
    ##  UploadBinary Or Text File (probably fixed as binary
    ##########################################################################

    def upload_files( self, target_url, params, files_list, xWidget=None, debugMode=False ):


        #print "======================= UPLOAD    =============="
        files = {}
        for f in files_list:
            fileInfo = QtCore.QFileInfo( f )
            if not fileInfo.exists():
                #print "'%s' not exist" % f
                return None
            else:
                files[f] = fileInfo.fileName()

        params['file_count'] = len( files )



        #xx = QtCore.QDateTime().currentDateTime().toString("hh_mm_ss_")
        #file_name =  xx + fileInfo.fileName() # QtCore.QDateTime().currentDateTime().toString("hh_mm_ss") + ".xls"

        #fileObj = QtCore.QFile(file_path)
        #fileObj.open(QtCore.QIODevice.ReadOnly)
        #fileObj = open(file_path, 'rb')
        #print fileObj.read_all()
        ### Constants
        CRLF = "\r\n"
        BOUNDARY = "-----------------------------7d935033608e2" #any string to mark boundaries
        START_DELIM = "--" + BOUNDARY + CRLF # delimiter for start of content "block"
        CONTENT_DISPOSITION = 'Content-Disposition: form-data; '

        ## Byte array to append to
        requestContent = QtCore.QByteArray()

        ## Add the paramaters as form-data eg {'foo': 'bar', 'bar': 'beer'}

        for p_key in params:
            st = QtCore.QString( START_DELIM + ( 'Content-Disposition: form-data; name="%s"' % p_key ) + CRLF + CRLF )
            st.append( "%s%s" % ( params[p_key], CRLF ) )
            requestContent.append( st )

        ## Append the file - binary
        count = 0
        for file_path in files:

            fileObj = QtCore.QFile( file_path )
            fileObj.open( QtCore.QIODevice.ReadOnly )

            st = QtCore.QString( START_DELIM + 'Content-Disposition: form-data; name="file_%s"; filename="%s";%s' % ( count, files[file_path], CRLF ) )

            mime_type, encoding = mimetypes.guess_type( file_path )
            content_type = "Content-Type: %s" % mime_type
            st.append( content_type + CRLF + CRLF )

            requestContent.append( st )
            requestContent.append( fileObj.readAll() )

            term = QtCore.QString( CRLF + "--" + BOUNDARY + "--" + CRLF )
            requestContent.append( term )
            count += 1



        #dataToSend = QtCore.QString(data).toAscii()
        self.emit( QtCore.SIGNAL( "payload_size" ), requestContent.size() ) #, payload

        ## Constuct Request

        #self.url = QtCore.QUrl( "%s/rpc%s" % ( self.srv_url, action_url ) )
        self.url = QtCore.QUrl( "http://%s/%s" % ( self.srv.ip, target_url ) )

        request = QtNetwork.QNetworkRequest()
        request.setUrl( self.url )
        request.setRawHeader( "Content-Type", 'multipart/form-data; boundary="%s"' % BOUNDARY )
        request.setRawHeader("Host", self.srv.host)
        request.setHeader( QtNetwork.QNetworkRequest.ContentLengthHeader, QtCore.QVariant( requestContent.size() ) )
        self.load_cookies()



    def file_upload_progress( self, progress, size ):
        self.emit( QtCore.SIGNAL( "uploadProgress" ), progress, size )







import platform



from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico
import G
import cwidgets



class SlaveDialog(QtWidgets.QDialog):

    sigSaved = pyqtSignal(dict)


    DATA_TYPE_ROLE = Qt.UserRole + 3

    def __init__(self, parent=None, dev_id=None, serverConn=None):
        super().__init__(parent)

        self.dev_id = dev_id
        self.serverConn = serverConn
        assert self.serverConn != None
        self.rec = None

        self.setWindowTitle("Slave Details")
        self.setWindowIcon(Ico.icon(Ico.slave))
        self.setMinimumWidth(800)



        self.mainLayout = cwidgets.vlayout(margin=20)
        self.setLayout(self.mainLayout)

        colLayout = cwidgets.hlayout(spacing=10)
        self.mainLayout.addLayout(colLayout)

        leftLay = cwidgets.vlayout()
        colLayout.addLayout(leftLay)

        self.grid = QtWidgets.QGridLayout()
        self.grid.setSpacing(10)
        leftLay.addLayout(self.grid)

        row = 0
        self.chkActive = QtWidgets.QCheckBox()
        self.chkActive.setText("Active")
        self.grid.addWidget(self.chkActive, row, 1)

        self.lblID = QtWidgets.QLabel()
        self.grid.addWidget(self.lblID, row, 2)

        row += 1
        self.grid.addWidget(QtWidgets.QLabel("Label / Name"), row, 0, Qt.AlignRight)
        self.txtDeviceName = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtDeviceName, row, 1)
        self.txtDeviceName.setPlaceholderText("eg RPI #1")

        row  += 1
        self.grid.addWidget(QtWidgets.QLabel("Type"), row, 0, Qt.AlignRight)
        self.cmbDeviceType = QtWidgets.QComboBox()
        self.grid.addWidget(self.cmbDeviceType, row, 1, 1, 2)
        self.cmbDeviceType.currentIndexChanged.connect(self.on_combo_dev_type)


        row += 1
        self.grid.addWidget(QtWidgets.QLabel("Slave ID"), row, 0, Qt.AlignRight)
        self.spinSlaveID = QtWidgets.QSpinBox()
        self.spinSlaveID.setRange(0, 100)
        self.grid.addWidget(self.spinSlaveID, row, 1)

        row += 1
        self.lblIpAddress = QtWidgets.QLabel("IP Address")
        self.grid.addWidget(self.lblIpAddress, row, 0, Qt.AlignRight)
        self.txtIPAddress = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtIPAddress, row, 1, 1, 2)
        #self.txtIPAddress.setInputMask("000.000.000.000;_")

        row += 1
        self.lblIpPort = QtWidgets.QLabel("Port")
        self.grid.addWidget(self.lblIpPort, row, 0, Qt.AlignRight)
        self.spinIpPort = QtWidgets.QSpinBox()
        self.spinIpPort.setRange(0, 65535)
        self.grid.addWidget(self.spinIpPort, row, 1)

        row += 1
        self.lblCommPort = QtWidgets.QLabel("Comm Port")
        self.grid.addWidget(self.lblCommPort , row, 0, Qt.AlignRight)
        self.cmbCommPort = QtWidgets.QComboBox()
        self.grid.addWidget(self.cmbCommPort, row, 1)
        self.cmbCommPort.setEditable(True)

        row += 1
        self.lblBaudRate = QtWidgets.QLabel("Baud Rate")
        self.grid.addWidget(self.lblBaudRate , row, 0, Qt.AlignRight)
        self.cmbBaudRate = QtWidgets.QComboBox()
        self.grid.addWidget(self.cmbBaudRate, row, 1)
        self.cmbBaudRate.setEditable(True)
        for b in [1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200]:
            self.cmbBaudRate.addItem(str(b))


        row += 1
        self.lblParity = QtWidgets.QLabel("Parity")
        self.grid.addWidget(self.lblParity , row, 0, Qt.AlignRight)
        self.cmbParity = QtWidgets.QComboBox()
        self.grid.addWidget(self.cmbParity, row, 1)
        for p in ["none", "odd", "even"]:
            self.cmbParity.addItem(p)


        row += 1
        self.lblDataBits = QtWidgets.QLabel("Data Bits")
        self.grid.addWidget(self.lblDataBits , row, 0, Qt.AlignRight)
        self.spinDataBits = QtWidgets.QSpinBox()
        self.spinDataBits.setRange(0, 256)
        self.grid.addWidget(self.spinDataBits, row, 1)


        row += 1
        self.lblStopBits = QtWidgets.QLabel("Stop Bits")
        self.grid.addWidget(self.lblStopBits , row, 0, Qt.AlignRight)
        self.spinStopBits = QtWidgets.QSpinBox()
        self.spinStopBits.setRange(0, 2)
        self.grid.addWidget(self.spinStopBits, row, 1)

        self.grid.setColumnStretch(0, 1)
        self.grid.setColumnStretch(1, 1)
        self.grid.setColumnStretch(2, 1)

        leftLay.addStretch(20)


        #========================
        rightLay = cwidgets.vlayout(spacing=15)
        colLayout.addLayout(rightLay)

        self.ioWidgets = {}

        for item in IO_PREFIXES:

            self.ioWidgets[item['prefix']] = IO_ItemWidget(prefix=item['prefix'], label=item['label'])
            rightLay.addWidget(self.ioWidgets[item['prefix']])


        self.actionBar = cwidgets.FormActionBar(self)
        self.mainLayout.addWidget(self.actionBar)

        self.fetch()




    def on_cancel(self):
        self.reject()

    def on_validate(self):

        s = self.txtServerAddress.s()
        url = QtCore.QUrl(s)
        ena = False
        if url.isValid() and url.scheme() in ["http", "https"]:
            ena = True
        self.buttPing.setEnabled(ena)

    def selected_conn_type(self):
        if self.cmbDeviceType.currentIndex() == 0:
            return None
        return self.cmbDeviceType.itemData(self.cmbDeviceType.currentIndex(), self.DATA_TYPE_ROLE)['conn_type']

    def on_save(self):



        for w in [self.txtDeviceName]:
            if len(w.s()) < 4:
                #w.borderInvalid()
                w.setFocus()
                return

        typ = self.selected_conn_type()
        if typ == None:
            self.cmbDeviceType.setFocus()
            return

        if typ == "net":
            if len(self.txtIPAddress.s()) < 4:
                self.txtIPAddress.setFocus()
                return
            if self.spinIpPort.value() < 10:
                self.spinIpPort.setFocus()
                return

        elif type == "serial":

            if self.cmbCommPort.len() < 4:
                self.cmbCommPort.setFocus()
                return


        nrec = dict(
            dev_id = self.dev_id,
            active = self.chkActive.isChecked(),
            dev_name = self.txtDeviceName.s(),
            dev_type = self.cmbDeviceType.currentData(),
            slave_id = self.spinSlaveID.value(),

            ip_address = self.txtIPAddress.s(),
            ip_port = self.spinIpPort.value(),

            com_port = self.cmbCommPort.currentText(),
            baud_rate = int(self.cmbBaudRate.currentText()),
            parity = self.cmbParity.currentText(),
            data_bits = self.spinDataBits.value(),
            stop_bits =self.spinStopBits.value(),

            conn_type = typ

        )

        for dt in IO_PREFIXES:
            dic = self.ioWidgets[dt['prefix']].get_values()
            nrec.update(dic)

        self.serverConn.post(self, "/slave-devices/%s" % self.dev_id, data=nrec)


    def fetch(self):
        self.serverConn.get(self, "/slave-devices/%s" % self.dev_id)

    def on_server_reply(self, reply):

        if reply.busy:
            return
        #print(reply)

        #print(reply.data)
        if reply.data and "error" in reply.data and reply.data['error']:
            print(reply.data)
            self.actionBar.status
            return

        if reply.post:
            print("<<<<<<<<<<<<<<<<<<<<<<<<<<")
            print("POST+", reply)
            if reply.error:
                dsadsa()
                return
            self.sigSaved.emit(reply.data['slave'])
            self.accept()

            #for k in reply.data['pre']:
            #    print(" " if reply.data['pre'][k] == reply.data['merge'][k] else "#", k, reply.data['pre'][k], reply.data['merge'][k], reply.data['src'][k])
            #pprint(reply.data['pre'])
            #pprint(reply.data['merge'])
            return

        self.rec = reply.data['slave']
        #for k in sorted(self.rec.keys()):
        #    print(k, self.rec[k])

        self.actionBar.set_meta("dev_id", self.rec)

        # Serial
        self.cmbCommPort.addItem("")
        for r in reply.data['serial_ports']:
            self.cmbCommPort.addItem(r)




        # Types
        self.cmbDeviceType.addItem("-- select --")
        for idx, dt in enumerate(reply.data['device_types']):
            self.cmbDeviceType.addItem(dt['label'], dt['dev_type'])
            self.cmbDeviceType.setItemData(idx + 1, dt, self.DATA_TYPE_ROLE)
        idx = self.cmbDeviceType.findData(self.rec['dev_type'])
        if idx != -1:
            self.cmbDeviceType.setCurrentIndex(idx)

        self.chkActive.setChecked(self.rec['active'] == True)

        self.lblID.setText(self.rec['dev_id'])
        self.txtDeviceName.setText(self.rec['dev_name'])

        self.txtIPAddress.setText(self.rec['ip_address'])
        self.spinIpPort.setValue(self.rec['ip_port'])

        self.spinStopBits.setValue(self.rec['stop_bits'])
        self.spinDataBits.setValue(self.rec['data_bits'])

        for dt in IO_PREFIXES:
            self.ioWidgets[dt['prefix']].set_values(self.rec)

    def show_serial_widgets(self, is_serial):

        self.lblIpAddress.setVisible(not is_serial)
        self.txtIPAddress.setVisible(not is_serial)

        self.lblIpPort.setVisible(not is_serial)
        self.spinIpPort.setVisible(not is_serial)

        self.lblCommPort.setVisible(is_serial)
        self.cmbCommPort.setVisible(is_serial)

        self.lblBaudRate.setVisible(is_serial)
        self.cmbBaudRate.setVisible(is_serial)

        self.lblParity.setVisible(is_serial)
        self.cmbParity.setVisible(is_serial)

        self.lblDataBits.setVisible(is_serial)
        self.spinDataBits.setVisible(is_serial)

        self.lblStopBits.setVisible(is_serial)
        self.spinStopBits.setVisible(is_serial)

    def show_ip_widgets(self, state):

        self.lblIpAddress.setVisible(state)
        self.txtIPAddress.setVisible(state)

        self.lblIpPort.setVisible(state)
        self.spinIpPort.setVisible(state)


    def on_combo_dev_type(self, idx=None):

        idx = self.cmbDeviceType.currentIndex()
        if idx == 0:
            self.show_serial_widgets(False)
            self.show_ip_widgets(False)
            return

        data = self.cmbDeviceType.itemData(idx, self.DATA_TYPE_ROLE)
        self.show_serial_widgets(data['conn_type'] == "serial")
        self.show_ip_widgets(data['conn_type'] == "net")







IO_PREFIXES = [
    {"prefix": "di", "label": "Discrete Inputs (%IX100.0)"},
    {"prefix": "coil", "label": "Coils (%QX100.0)"},
    {"prefix": "ir", "label": "Input Registers (%IW100)"},
    {"prefix": "hr_read", "label": "Holding Registers - Read (%IW100)"},
    {"prefix": "hr_write", "label": "Holding Registers - Write (%QW100)"},
]

class IO_ItemWidget(QtWidgets.QGroupBox):


    def __init__(self, parent=None, prefix=None, label=None):
        super().__init__(parent)

        self.prefix = prefix
        self.setTitle(label)



        self.grid = QtWidgets.QGridLayout()
        self.grid.setSpacing(10)
        self.setLayout(self.grid)

        row = 0
        self.grid.addWidget(QtWidgets.QLabel("Start Address"), row, 0)

        self.spinStart = QtWidgets.QSpinBox()
        self.grid.addWidget(self.spinStart, row, 1)

        self.grid.addWidget(QtWidgets.QLabel("Size"), row, 2)

        self.spinSize = QtWidgets.QSpinBox()
        self.grid.addWidget(self.spinSize, row, 3)


    def set_values(self, rec):

        v = rec["%s_start" % self.prefix]
        self.spinStart.setValue(v)

        v = rec["%s_size" % self.prefix]
        self.spinSize.setValue(v)

    def get_values(self):
        dic = {}
        dic["%s_start" % self.prefix] = self.spinStart.value()
        dic["%s_size" % self.prefix] = self.spinSize.value()
        return dic











from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico

from server import server_conn, server_dialog
from slave_devices import slave_devices_grid, modbus_widgets
from users import users_grid
from programs import programs_grid
from network import websocket_panel
from runtime import settings_dialog, runtime_commander, ssh_terminal
#from runtime import pyqterm_test

class ServerPanel(QtWidgets.QMainWindow):



    def __init__(self, parent=None, server=None):
        QtWidgets.QMainWindow.__init__(self, parent)

        self.server = server
        self.serverConn = server_conn.ServerConn(self, server_address=server["address"])





        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.addToolBar(Qt.TopToolBarArea, self.toolbar)

        # Creds -------------------------
        self.tbServer = cwidgets.ToolBarGroup(self, title="Server Connection")
        self.toolbar.addWidget(self.tbServer)

        self.lblServerName = cwidgets.XLabel()
        self.lblServerName.setFixedWidth(160)
        self.tbServer.addWidget(self.lblServerName)

        self.lblServerStatus = cwidgets.XLabel()
        self.lblServerStatus.setFixedWidth(160)
        self.tbServer.addWidget(self.lblServerStatus)

        self.buttServer = cwidgets.XToolButton(self, ico=Ico.server, both=False, callback=self.on_server_edit)
        self.tbServer.addWidget(self.buttServer)

        self.buttWs = cwidgets.XToolButton(self, ico=Ico.save, both=False, callback=self.on_ws)
        self.tbServer.addWidget(self.buttWs)

        self.toolbar.addSeparator()


        # Runtime
        self.tbRuntime = cwidgets.ToolBarGroup(self, title="OpenPLC Runtime", is_group=True, toggle_icons=False)
        self.toolbar.addWidget(self.tbRuntime)

        self.buttStop = cwidgets.XToolButton(self, ico=Ico.stop, text="Stop", callback=self.on_runtime_stop)
        self.tbRuntime.addWidget(self.buttStop)

        self.buttStart = cwidgets.XToolButton(self, ico=Ico.start, text="Start", callback=self.on_runtime_start)
        self.tbRuntime.addWidget(self.buttStart)


        self.lblRuntimeStatus = cwidgets.XLabel("Status")
        self.tbRuntime.addWidget(self.lblRuntimeStatus)

        self.toolbar.addSeparator()

        # Actions
        self.tbActions = cwidgets.ToolBarGroup(self, title="Actions")
        self.toolbar.addWidget(self.tbActions)

        self.buttSettings = cwidgets.XToolButton(self, ico=Ico.settings, text="Settings", callback=self.on_settings)
        self.tbActions.addWidget(self.buttSettings)

        self.buttMbConfig = cwidgets.XToolButton(self, ico=Ico.mbconfig, text="mbconfig.cfg", callback=self.on_mbconfig)
        self.tbActions.addWidget(self.buttMbConfig)

        self.mainWidget = QtWidgets.QWidget()
        self.mainLayout = cwidgets.vlayout()
        self.mainWidget.setLayout(self.mainLayout)
        self.setCentralWidget(self.mainWidget)

        self.mainLayout.addSpacing(10)


        ## Midx widgets
        self.tabWidget = QtWidgets.QTabWidget()
        self.mainLayout.addWidget(self.tabWidget)


        #self.pyQTerm = pyqterm_test.PyQTermWidget(self, serverConn=self.serverConn)
        #self.tabWidget.addTab(self.pyQTerm, Ico.icon(Ico.terminal), "PyQTermWidget")

        self.sshTerminal = ssh_terminal.SshTerminalWidget(self, serverConn=self.serverConn)
        self.tabWidget.addTab(self.sshTerminal, Ico.icon(Ico.terminal), "SSH Terminal")

        self.runCommander = runtime_commander.RuntimeCommanderWidget(self, serverConn=self.serverConn)
        self.tabWidget.addTab(self.runCommander, Ico.icon(Ico.commander), "Commander")

        self.programsGrid = programs_grid.ProgramsGrid(self, serverConn=self.serverConn)
        self.tabWidget.addTab(self.programsGrid, Ico.icon(Ico.programs), "Programs")

        self.usersGrid = users_grid.UsersGrid(self, serverConn=self.serverConn)
        self.tabWidget.addTab(self.usersGrid, Ico.icon(Ico.users), "Users")


        self.slavesGrid = slave_devices_grid.SlaveDevicesGrid(self, serverConn=self.serverConn)
        self.tabWidget.addTab(self.slavesGrid, Ico.icon(Ico.slaves), "Slaves")

        self.lblServerName.setText("-")
        self.lblServerStatus.setText("-")


        ## Docks
        dock = QtWidgets.QDockWidget("Web Socket")
        self.addDockWidget(Qt.LeftDockWidgetArea, dock)
        self.webSockPanel = websocket_panel.WebSocketPanel(self, serverConn=self.serverConn)
        dock.setWidget(self.webSockPanel)


    def on_ws(self):
        #self.serverConn.connect_websocket()

        self.serverConn.connect_websocket()


    def init_load(self):

        # self.fetch()
        self.on_login()

    def on_login(self):
        creds = dict(login=self.server['login'], password=self.server['password'])
        self.serverConn.post(self, "/login", data=creds, tag="login")


    def on_server_reply(self, reply):

        if reply.error:

            self.lblServerStatus.setText(reply.error)
            return
            self.statusBar.set_reply(reply)

        if reply.tag == "login":

            self.lblServerName.setText("-")
            self.lblServerStatus.setText("-")

            if reply.data and "logged_in" in reply.data and reply.data["logged_in"] == True:
                self.on_logged_in()
                self.lblServerName.setText(self.server['address'])
                self.lblServerStatus.setText("Connected")

                # &*^&^^&*^^**
                #self.serverConn.connect_websocket()

        if reply.tag == "runtime":

            self.lblServerName.setText("-")
            self.lblServerStatus.setText("-")

            self.serverConn.connect_runtime()




    def on_mbconfig(self):
        dial = modbus_widgets.MbConfigDialog(self, serverConn=self.serverConn)
        dial.exec_()

    def on_logged_in(self):
        for idx in range(0, self.tabWidget.count()):
            self.tabWidget.widget(idx).init_load()

    def on_server_edit(self):
        dial = server_dialog.ServerDialog(self, server=self.server)
        dial.exec_()


    def on_settings(self):
        dial = settings_dialog.SettingsDialog(self, serverConn=self.serverConn)
        dial.exec_()


    def on_runtime_start(self):
        print("start")
        self.serverConn.post(self, "/runtime/start", tag="runtime")

    def on_runtime_stop(self):
        print("stop")
        self.serverConn.post(self, "/runtime/stop", tag="runtime")


# -*- coding: utf-8 -*-

## Custom and Extended widgets

from Qt import QtCore, QtGui, QtWidgets, Qt, pyqtSignal
from img import Ico

DEFAULT_SPACING = 0
DEFAULT_MARGIN = 0
DEFAULT_BUTTON_WIDTH = 80



#=====================================================
# Layouts

def hlayout(spacing=DEFAULT_SPACING, margin=DEFAULT_MARGIN):
    """Convenience function to create a QHBoxLayout"""
    lay = QtWidgets.QHBoxLayout()
    if isinstance(margin, bool):
        margin = DEFAULT_SPACING
    if isinstance(spacing, bool):
        spacing = DEFAULT_SPACING
    lay.setContentsMargins(margin, margin, margin, margin)
    lay.setSpacing(spacing)
    return lay

def vlayout(spacing=DEFAULT_SPACING, margin=DEFAULT_MARGIN):
    """Convenience function to create a QVBoxLayout"""
    lay = QtWidgets.QVBoxLayout()
    if isinstance(margin, bool):
        margin = DEFAULT_SPACING
    if isinstance(spacing, bool):
        spacing = DEFAULT_SPACING
    lay.setContentsMargins(margin, margin, margin, margin)
    lay.setSpacing(spacing)
    return lay


class CancelButton( QtWidgets.QPushButton ):
    def __init__( self, parent, text="Cancel", callback=None ):
        QtWidgets.QPushButton.__init__( self, parent )

        self.setText( text )
        self.setIcon( Ico.icon( Ico.cancel ) )
        if callback:
            self.clicked.connect(callback)
            return
        self.clicked.connect(parent.on_cancel)

class SaveButton( QtWidgets.QPushButton ):
    def __init__( self, parent, text="Save", ico=None, callback=None, width=None):
        super().__init__( parent )

        if width:
            self.setMinimumWidth( width )
        self.setText( text )
        self.setIcon( Ico.icon( ico if ico else Ico.save ) )

        if callback:
            self.clicked.connect(callback)
        else:
            self.clicked.connect(parent.on_save)

class DeleteButton( QtWidgets.QPushButton ):
    def __init__( self, parent, text="Delete", callback=None ):
        QtWidgets.QPushButton.__init__( self, parent )

        self.setText( text )
        self.setIcon( Ico.icon( Ico.delete ) )
        if callback:
            self.clicked.connect(callback)
            return
        self.clicked.connect(parent.on_delete)



class FormActionBar(QtWidgets.QWidget):

    def __init__(self, parent, delete=False, show_meta=True):
        super().__init__(parent)

        self.show_meta = show_meta

        self.mainLayout = vlayout(margin=5, spacing=10)
        self.setLayout(self.mainLayout)



        #---
        self.buttLay = hlayout(spacing=5)
        self.mainLayout.addLayout(self.buttLay)

        if delete:
            self.buttDelete = DeleteButton(parent)
            self.buttLay.addWidget(self.buttDelete)

        self.statusBar = StatusBar(self, refresh=False)
        self.buttLay.addWidget(self.statusBar)

        self.buttCancel = CancelButton(parent)
        self.buttLay.addWidget(self.buttCancel)


        self.buttSave = SaveButton(parent)
        self.buttLay.addWidget(self.buttSave)



        # ---
        if show_meta:
            self.metaLay = hlayout(spacing=5)
            self.mainLayout.addLayout(self.metaLay)

            self.lblIDName = QtWidgets.QLabel()
            self.metaLay.addWidget(self.lblIDName)
            self.lblID = QtWidgets.QLabel()
            self.metaLay.addWidget(self.lblID)
            self.lblID.setFrameStyle(QtWidgets.QFrame.Sunken | QtWidgets.QFrame.Panel)

            self.metaLay.addWidget(QtWidgets.QLabel("Created:"))
            self.lblCreated = QtWidgets.QLabel()
            self.metaLay.addWidget(self.lblCreated)
            self.lblCreated.setFrameStyle(QtWidgets.QFrame.Sunken | QtWidgets.QFrame.Panel)

            self.metaLay.addWidget(QtWidgets.QLabel("Updated:"))
            self.lblUpdated = QtWidgets.QLabel()
            self.metaLay.addWidget(self.lblUpdated)
            self.lblUpdated.setFrameStyle(QtWidgets.QFrame.Sunken | QtWidgets.QFrame.Panel)

            self.metaLay.addStretch(2)


    def set_meta(self, id_fld, rec):
        if not self.show_meta: # should never appen
            return
        self.lblIDName.setText(id_fld)
        self.lblID.setText(rec[id_fld])
        self.lblCreated.setText("%s" % rec["date_created"])
        self.lblUpdated.setText("%s" % rec["date_updated"])


    def set_reply(self, reply):
        self.statusBar.set_reply(reply)



class XToolButton( QtWidgets.QToolButton ):

    def __init__( self, parent=None, colors=None, style=None,
                    autoRaise=True, menu=None,
                    text="", tooltip=None, ico=None, iconTop=False, iconSize=16,
                    bold=False, disabled=False,
                    width=None, popup=False,
                    callback=None, ki=None,
                    both=True, checkable=False
                    ):

        QtWidgets.QToolButton.__init__(self, parent)

        self.colors = colors


        if ki:
            self.setProperty("ki", ki)

        if width:
            self.setFixedWidth(width)

        if style:
            self.setStyleSheet(style)

        if disabled:
            self.setDisabled(True)


        if both:
            if iconTop:
                self.setToolButtonStyle( Qt.ToolButtonTextUnderIcon)
            else:
                self.setToolButtonStyle( Qt.ToolButtonTextBesideIcon)
        else:
            self.setToolButtonStyle( Qt.ToolButtonIconOnly)

        if checkable:
            self.setCheckable(True)

        if callback:
            self.clicked.connect(callback)

        if tooltip:
            self.setToolTip(tooltip)

        if text:
            self.setText(text)

        #== Font
        # Try and avoid styleSheet maybe
        if bold:
            self.setBold(bold)
        # < font

        if ico:
            self.setIco(ico)
            if iconSize:
                if isinstance(iconSize, list):
                    self.setIconSize( QtCore.QSize(iconSize[0],iconSize[0]))
                else:
                    self.setIconSize( QtCore.QSize(iconSize, iconSize))

        self.setAutoRaise(autoRaise)
        if popup:
            self.setPopupMode(QtWidgets.QToolButton.InstantPopup)

        if menu:
            self.setMenu(QtWidgets.QMenu())

    def setIco(self, ico, iconSize=16):
        self.setIcon(Ico.icon(ico))
        self.setIconSize( QtCore.QSize(iconSize, iconSize))

    def setEnabled(self, state):
        if self.colors:
            color = self.colors[1] if state else self.colors[0]
            self.setStyleSheet("background-color: %s" % color)
        QtWidgets.QToolButton.setEnabled(self, state)

    def setBold(self, state):
        f = self.font()
        f.setBold(state)
        self.setFont(f)



C_FG = "color: #222222;"
C_BG = "background-color: white;"

C_FG_FOCUS = "color: black;"
C_BG_FOCUS = "background-color: #FFFA93"



class XLineEdit( QtWidgets.QLineEdit ):

    sigFocused = pyqtSignal(bool)
    sigMove = pyqtSignal(bool)
    sigDoubleClicked = pyqtSignal()

    def __init__( self, parent=None, show_focus=False, width=None,
                  dirty=True, changed=None ):
        QtWidgets.QLineEdit.__init__(self, parent )

        self._show_focus = show_focus

        if width:
            self.setFixedWidth(width)

        if dirty:
            if hasattr(parent, "set_dirty"):
                self.textChanged.connect(parent.set_dirty)
        if changed:
            self.textChanged.connect(changed)

    def stripped(self):
        self.setText( self.text().strip() )
        return self.text()

    def s(self):
        return self.stripped()

    def len(self):
        return len(self.text().strip())

    def setText(self, txt):
        if txt == None:
            self.setText("")
            return
        super().setText(txt.strip())

    def set_bold(self, state):
        f = self.font()
        f.setBold(state)
        self.setFont(f)

    def mouseDoubleClickEvent_MAYBE(self, ev):
        ev.ignore()
        self.sigDoubleClicked.emit(ev)

    def focusInEvent(self, ev):
        """Changes style if show_focus """
        if self._show_focus:
            self.setStyleSheet(C_FG_FOCUS + C_BG_FOCUS)
        QtWidgets.QLineEdit.focusInEvent(self, ev)
        self.sigFocused.emit(True)

    def focusOutEvent(self, ev):
        """Changes style if show_focus """
        if self._show_focus:
            self.setStyleSheet(C_FG + C_BG)

        QtWidgets.QLineEdit.focusOutEvent(self, ev)
        self.sigFocused.emit(False)


    def keyPressEvent(self, ev):
        """Clear field with esc, otherwise passthough"""
        if ev.key() == Qt.Key_Escape:
            self.setText("")
            return
        if ev.key() == Qt.Key_Up:
            self.sigMove.emit(False)
            return
        if ev.key() == Qt.Key_Down:
            self.sigMove.emit(True)
            return
        QtWidgets.QLineEdit.keyPressEvent( self, ev )

class XToolBar(QtWidgets.QToolBar):

    def __init__( self, parent=None):
        QtWidgets.QToolBar.__init__(self, parent)

    def addStretch(self):

        widget = QtWidgets.QWidget()
        sp = QtWidgets.QSizePolicy()
        sp.setHorizontalPolicy(QtWidgets.QSizePolicy.Expanding)
        widget.setSizePolicy(sp)

        self.addWidget(widget)


class ToolBarGroup(QtWidgets.QWidget):
    def __init__(self, parent=None, title=None, width=None, hide_labels=False, bg='#999999',
                 is_group=False, toggle_icons=False, toggle_callback=None):
        QtWidgets.QWidget.__init__(self, parent)

        if width:
            self.setFixedWidth(width)

        self.icon_on = Ico.filter_on
        self.icon_off = Ico.filter_off
        self.toggle_icons = toggle_icons
        self.toggle_callback = toggle_callback
        self.hide_labels = hide_labels

        self.buttonGroup = None
        self.is_group = is_group
        if self.is_group:
            self.buttonGroup = QtWidgets.QButtonGroup()
            self.buttonGroup.setExclusive(True)
            if self.toggle_callback:
                self.buttonGroup.buttonClicked.connect(self.on_button_clicked)

        self.group_var = None
        self.callback = None
        self.show_icons = True
        self.icon_size = 12
        self.bg_color = bg

        ## Main Layout
        mainLayout = QtWidgets.QVBoxLayout()
        mainLayout.setContentsMargins(0, 0, 0, 0)
        mainLayout.setSpacing(0)
        self.setLayout(mainLayout)

        ## Label
        self.label = QtWidgets.QLabel()
        #bg = "#8F8F8F"  ##eeeeee"
        fg = "#eeeeee"  ##333333"
        lbl_sty = "background: %s; " % self.bg_color  # qlineargradient(x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #fefefe, stop: 1 #CECECE);"
        lbl_sty += " color: %s; font-size: 8pt; padding: 1px;" % fg  # border: 1px outset #cccccc;"
        self.label.setStyleSheet(lbl_sty)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setFixedHeight(20)
        mainLayout.addWidget(self.label)

        ## Toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolbar.setFixedHeight(30)

        mainLayout.addWidget(self.toolbar)

        if title:
            self.set_title(title)
        #print(ere)
    def set_title(self, title):
        self.label.setText("%s" % title)

    def addWidget(self, widget):
        self.toolbar.addWidget(widget)
        return widget

    def addAction(self, act):
        self.toolbar.addAction(act)

    def addButton(self, ico=None, text=None, callback=None, idx=None, toggle_callback=None, tooltip=None,
                  ki=None, bold=False, checkable=False, checked=None, width=None, return_action=False):

        butt = XToolButton()

        if self.is_group:
            if idx != None:
                self.buttonGroup.addButton(butt, idx)
            else:
                self.buttonGroup.addButton(butt)
        if self.hide_labels == False:
            if text != None:
                butt.setText(text)
        if text == None:
            butt.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        else:
            butt.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        if tooltip:
            butt.setToolTip(tooltip)

        if self.toggle_icons:
            butt.setIconSize(QtCore.QSize(10, 10))
            butt.setIcon(Ico.icon(self.icon_off))
        if ico:
            butt.setIcon(Ico.icon(ico))
            butt.setIconSize(QtCore.QSize(10, 10))

        butt.setCheckable(checkable)
        if checked != None:
            butt.setChecked(checked)


        butt.setProperty("ki", ki)
        nuAct = self.toolbar.addWidget(butt)
        if callback:
            self.connect(butt, QtCore.SIGNAL("clicked()"), callback)
        #if toggle_callback:
        #   self.connect(butt, QtCore.SIGNAL("toggled(bool)"), toggle_callback)
        if bold:
            self.set_bold(butt)
        if width:
            butt.setFixedWidth(width)

        self.on_button_clicked(block=True)

        if return_action:
            return nuAct
        return butt

    def set_bold(self, w):
        f = w.font()
        f.setBold(True)
        w.setFont(f)

    def on_button_clicked(self, butt=None, block=False):
        if self.is_group:
            for b in self.buttonGroup.buttons():
                b.setIcon( Ico.icon(self.icon_on if b.isChecked() else self.icon_off) )

                if block == False and b.isChecked():
                    if self.toggle_callback:
                        self.toggle_callback(self.buttonGroup.id(b))

    def get_id(self):
        id = self.buttonGroup.checkedId()
        if id == -1:
            return None
        return id



class XLabel(QtWidgets.QLabel):

    sigClicked = pyqtSignal()
    sigDoubleClicked = pyqtSignal()

    def __init__( self, parent=None, bold=False,
                  style=None, base_style="", align=None,
                  text=None, frame=None,
                  wrap=False, hover_color=None,
                  height=None, width=None):
        QtWidgets.QLabel.__init__( self, parent )

        self.hover_color = hover_color

        self.base_style = base_style
        self.style = ""
        if style != None:
            self.style = style
        self.setStyleSheet(self.style)

        if bold:
            f = self.font()
            f.setBold(True)
            self.setFont(f)

        if align != None:
            self.setAlignment(align)

        if text:
            self.setText(text)

        if height:
            self.setFixedHeight(height)
        if width:
            self.setFixedWidth(width)

        self.setWordWrap(wrap)

        if frame:
            self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Raised)

        if self.hover_color:
            self.set_border("#dddddd")

    def setStyleSheet(self, sty):
        QtWidgets.QLabel.setStyleSheet(self, sty + self.base_style)



class StatusBar(QtWidgets.QWidget):
    """A QWidget with many embedded widgets for StatusBar"""

    sigRefresh = pyqtSignal()

    def __init__(self, parent, refresh=True, status=True,
                 pager=False, mode=True):
        QtWidgets.QWidget.__init__(self, parent)

        if isinstance(parent, bool):
            print("STATUSBar is not correct parent")
            # print Tantrum
            return

        self._lastReply = None

        self._refresh = refresh
        self._status = status

        self.mainLayout = QtWidgets.QHBoxLayout()
        m = 0
        self.mainLayout.setContentsMargins(m, m, m, m)
        self.setLayout(self.mainLayout)


        ##=======================================================================
        ## Status Bar
        self.statusBar = QtWidgets.QStatusBar()
        self.statusBar.setSizeGripEnabled(False)
        self.showMessage(" ")
        self.mainLayout.addWidget(self.statusBar, 1)

        ## Main layout to add
        self.extrasLayout = QtWidgets.QHBoxLayout()
        self.mainLayout.addLayout(self.extrasLayout, 0)

        ## Status widget setup

        self.lblStatus = QtWidgets.QLabel()
        self.lblStatus.setFixedWidth(30)
        self.lblStatus.setStyleSheet("background-color: yellow;")
        self.mainLayout.addWidget(self.lblStatus)


        #== Progress Bar
        progressWidget = QtWidgets.QWidget()
        progressWidget.setFixedWidth(40)
        #progressWidget.setMaximumWidth(200)
        self.mainLayout.addWidget(progressWidget)

        progressLayout = QtWidgets.QHBoxLayout()
        m = 0
        progressLayout.setContentsMargins(m, m, m, m)
        progressWidget.setLayout(progressLayout)

        self.progress = QtWidgets.QProgressBar(self)
        # self.progress.setFixedWidth( 40 )
        self.progress.setFixedHeight(15)
        # self.setContentsMargins( m, m, m, m )
        self.progress.setMinimum(0)
        self.progress.setMaximum(1)
        self.progress.setInvertedAppearance(False)
        self.progress.setTextVisible(False)
        self.progress.setVisible(False)
        # self.connect( self.progress, QtCore.SIGNAL('valueChanged(int)'), self.on_progress_value_changed)
        progressLayout.addWidget(self.progress)


        ################################################################
        ## Refrehsing Button
        self.buttRefresh = None
        self.popMenu = None
        if refresh:
            self.buttRefresh = QtWidgets.QToolButton()
            self.mainLayout.addWidget(self.buttRefresh)

            self.buttRefresh.setIcon(Ico.icon(Ico.refresh))
            # self.buttRefresh.setFlat(True)
            self.buttRefresh.setAutoRaise(True)
            self.buttRefresh.setIconSize(QtCore.QSize(16, 16))
            self.buttRefresh.setStyleSheet("padding: 0px;")
            self.buttRefresh.clicked.connect(self.on_refresh)
            if hasattr(parent, "on_refresh"):
                self.buttRefresh.clicked.connect(parent.on_refresh)




    def on_refresh(self):
        self.sigRefresh.emit()



    ###################################################################
    def showMessage(self, mess, timeout=None, warn=None, info=None):
        # print "showMessage=", mess
        if warn:
            self.statusBar.setStyleSheet("color: #990000;")
        elif info:
            self.statusBar.setStyleSheet("color: #3361C2;")
        else:
            self.statusBar.setStyleSheet("")

        if timeout == None:
            self.statusBar.showMessage(mess)
        else:
            self.statusBar.showMessage(mess, timeout)

    def addWidget(self, widget):
        self.extrasLayout.addWidget(widget)

    def addPermanentWidget(self, widget):
        self.mainLayout.addWidget(widget)

    def insertPermanentWidget(self, idx, widget):
        self.statusBar.insertPermanentWidget(idx, widget)


    def set_status(self, sta):
        self.lblStatus.setText(sta.status)
        if sta.busy == False:
            self.progress_stop()
            return

        self.progress_start()

    def set_reply(self, reply):
        self.lblStatus.setText(reply.status)
        if reply.busy:
            self.progress_start()
            return

        self.progress_stop()
        self._reply = reply
        if reply.error:
            self.showMessage(reply.error, warn=True)

    def progress_start(self):
        self.progress.setMaximum(0)
        self.progress.setVisible(True)
        if self.buttRefresh:
            self.buttRefresh.setDisabled(True)

    def progress_stop(self):
        self.progress.setVisible(False)
        self.progress.setMaximum(1)
        if self.buttRefresh:
            self.buttRefresh.setDisabled(False)

    def set_busy(self, state):
        if state:
            self.progress_start()
        else:
            self.progress_stop()





# -*- coding: utf-8 -*-

import paramiko
from Qt import QtCore, QtWidgets, QtNetwork, Qt, pyqtSignal

from img import Ico

import cwidgets

from runtime import terminal_widget

class SshTerminalWidget(QtWidgets.QWidget):

    def __init__(self, parent, serverConn):
        QtWidgets.QWidget.__init__(self, parent)

        assert serverConn != None
        self.serverConn = serverConn

        self.sshClient = paramiko.SSHClient()
        self.sshClient.load_system_host_keys()
        self.sshClient.set_missing_host_key_policy(paramiko.WarningPolicy)



        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        self.toolbar = cwidgets.XToolBar()
        self.mainLayout.addWidget(self.toolbar)

        ## ------------ credentials
        grp = cwidgets.ToolBarGroup(title="Terminal")
        self.toolbar.addWidget(grp)

        self.txtHost = cwidgets.XLineEdit()
        grp.addWidget(self.txtHost)
        self.txtHost.setText("192.168.1.222")
        self.txtHost.setMaximumWidth(120)

        self.spinPort = QtWidgets.QSpinBox()
        grp.addWidget(self.spinPort)
        self.spinPort.setValue(22)

        self.txtUser = cwidgets.XLineEdit()
        grp.addWidget(self.txtUser)
        self.txtUser.setText("pi")
        self.txtUser.setMaximumWidth(80)

        self.txtPass = cwidgets.XLineEdit()
        grp.addWidget(self.txtPass)
        self.txtPass.setText("raspberry")
        self.txtPass.setMaximumWidth(80)

        self.buttConnect = cwidgets.XToolButton(ico=Ico.start, text="Connect", callback=self.on_connect)
        grp.addWidget(self.buttConnect)

        self.lblStatus = cwidgets.XLabel()
        grp.addWidget(self.lblStatus)



        ## ------------ send command
        grp = cwidgets.ToolBarGroup(title="Execute")
        self.toolbar.addWidget(grp)

        self.txtCommand = cwidgets.XLineEdit()
        grp.addWidget(self.txtCommand)
        self.txtCommand.setText("ls -l")
        self.txtCommand.returnPressed.connect(self.on_execute)

        self.buttConnect = cwidgets.XToolButton(ico=Ico.return_key, text="Execute", callback=self.on_execute)
        grp.addWidget(self.buttConnect)



        self.terminal = terminal_widget.Terminal(self)
        self.mainLayout.addWidget(self.terminal)

        ## ------------ tree
        self.model = LinesModel(self)

        #self.tree = QtWidgets.QTreeView()
        #self.mainLayout.addWidget(self.tree)
        #self.tree.setModel(self.model)

        #self.txt = QtWidgets.QTextEdit()
        #self.mainLayout.addWidget(self.txt)



        self.serverConn.sigSSHMessage.connect(self.on_ssh_message)

        self.terminal.add()

    def init_load(self):
        pass


    def deadon_connect(self):
        self.serverConn.connect_ssh(self.txtHost.s(), self.spinPort.value(),
                                    self.txtUser.s(), self.txtPass.s())

    def on_ssh_message(self, mess, err):
        #print(mess)
        print(err, len(err))

        if err:
            self.model.add_rec(2, err)
            self.txt.setPlainText(err)
            return
        self.txt.setPlainText(mess)
        self.model.add_rec(1, mess)



    def on_execute(self):
        cmd = self.txtCommand.s()
        #self.serverConn.send_ssh(cmd)
        self.terminal.run(cmd)



    def on_connect(self):
        # print("on_ssh")
        self.sshClient.connect(self.txtHost.s(), self.spinPort.value(),
                                    self.txtUser.s(), self.txtPass.s())
        self.send_ssh("uname -a")
        self.send_ssh("whoami")

    def send_ssh(self, cmd):
        stdin, stdout, stderr = self.sshClient.exec_command(cmd)
        # print( stdout.read(), )
        sout = str(stdout.read(), encoding="ascii")
        serr = str(stderr.read(), encoding="ascii")
        self.model.add_rec(0, serr)


class LinesModel(QtCore.QAbstractTableModel):

    def __init__(self, parent=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        self.lines = []
        self.ltypes = []

    def data(self, midx, role):
        cidx = midx.column()

        if role == Qt.DisplayRole:
            if cidx == 0:
                return "^" if self.ltypes[midx.row()] else ">"

            elif cidx == 1:
                return self.lines[midx.row()]


    def columnCount(self, pidx=None):
        return 2

    def rowCount(self, pidx=None):
        return len(self.lines)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["Exec", "RESP"][idx]

    def add_rec(self, typ, recv_lines):

        #lst = [s.strip() for s in recv_lines.split("\n")]
        for l in [s.strip() for s in recv_lines.split("\n")]:
            if l:
                self.ltypes.append(typ)
                self.lines.append(l)
        self.layoutChanged.emit()









from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico

from users import users_model
from users import user_dialog


class UsersGrid(QtWidgets.QWidget):

    def __init__(self, parent, serverConn=None):
        QtWidgets.QWidget.__init__(self, parent)

        self.serverConn = serverConn
        assert self.serverConn != None

        self.model = users_model.UsersModel(self, serverConn=self.serverConn)
        self.proxyModel = QtCore.QSortFilterProxyModel()
        self.proxyModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.proxyModel.setSourceModel(self.model)

        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        #===================
        # Toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.mainLayout.addWidget(self.toolbar)

        # User Actions
        grp = cwidgets.ToolBarGroup(self, "User Actions")
        self.toolbar.addWidget(grp)

        self.actionAdd = cwidgets.XToolButton(text="Add", ico=Ico.add, callback=self.on_add)
        grp.addWidget(self.actionAdd)

        self.actionEdit = cwidgets.XToolButton(text="Edit", ico=Ico.edit, callback=self.on_edit)
        grp.addWidget(self.actionEdit)
        self.actionEdit.setDisabled(True)


        # ===================
        # Tree
        self.tree = QtWidgets.QTreeView()
        self.mainLayout.addWidget(self.tree)

        self.tree.setModel(self.proxyModel)
        self.tree.setSortingEnabled(True)

        self.tree.selectionModel().selectionChanged.connect(self.on_tree_selection)
        self.tree.doubleClicked.connect(self.on_tree_double_clicked)



        self.statusBar = cwidgets.StatusBar(self)
        self.mainLayout.addWidget(self.statusBar)



    def init_load(self):

        self.fetch()

    def on_refresh(self):
        self.fetch()

    def fetch(self):
        self.model.fetch()



    def on_add(self):
        self.edit_user("new")

    def on_edit(self):
        if not self.tree.selectionModel().hasSelection():
            return

        pidx = self.tree.selectionModel().selectedIndexes()[0]
        sidx = self.proxyModel.mapToSource(pidx)
        tidx = self.model.index(sidx.row(), users_model.C.user_id)
        user_id = self.model.data(tidx, Qt.DisplayRole)
        print(user_id)
        self.edit_user(user_id)

    def edit_user(self, user_id):
        dial = user_dialog.UserDialog(self, serverConn=self.serverConn, user_id=user_id)
        dial.sigSaved.connect(self.on_saved)
        dial.exec_()

    def on_saved(self, rec):
        print(rec)
        ss



    def on_tree_selection(self, sel=None, desel=None):
        self.actionEdit.setEnabled(self.tree.selectionModel().hasSelection())

    def on_tree_double_clicked(self, midx):
        self.on_edit()








# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico
import G

class C:
    name = 0
    address = 1
    login = 2
    autoconnect = 3

    _count = 4


class ServersModel(QtCore.QAbstractTableModel):

    sigLoaded = pyqtSignal()

    def data(self, midx, role):

        rec = self.recs[midx.row()]
        cidx = midx.column()

        if role == Qt.DisplayRole:
            if cidx == C.name:
                return rec["name"]

            elif cidx == C.address:
                return rec['address']

            elif cidx == C.login:
                return rec['login']

            elif cidx == C.autoconnect:
                return rec['auto_connect']

        if role == Qt.DecorationRole and  cidx == C.name:
            return Ico.icon(Ico.server)



    def __init__(self, parent=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        self.recs = []


    def columnCount(self, pidx=None):
        return C._count

    def rowCount(self, pidx=None):
        return len(self.recs)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["Name", "Address", "Login", "Auto Connect"][idx]


    def load(self):

        self.recs = G.settings.servers_list()
        self.modelReset.emit()


# -*- coding: utf-8 -*-

import os

## App Globals

HERE_PATH =  os.path.abspath(os.path.dirname(__file__))
ROOT_PATH = os.path.abspath(os.path.join(HERE_PATH, ".."))

STATIC_PATH = os.path.abspath(os.path.join(ROOT_PATH, "static"))

print("HERE_PATH=", HERE_PATH)
print("ROOT_PATH=", ROOT_PATH)

import settings as settings_lib


settings = settings_lib.XSettings()





# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico

import cwidgets



class ProgramDialog(QtWidgets.QDialog):

    sigSaved = pyqtSignal(dict)
    sigDeleted = pyqtSignal(str)

    @property
    def url(self):
        return "/programs/%s" % self.prog_id

    def __init__(self, parent=None, serverConn=None, prog_id=None):
        super().__init__(parent)

        assert serverConn != None
        assert prog_id != None

        self.serverConn = serverConn
        self.prog_id = prog_id

        self.setWindowTitle("User")
        self.setWindowIcon(Ico.icon(Ico.user))
        self.setMinimumWidth(500)



        self.mainLayout = cwidgets.vlayout(margin=20)
        self.setLayout(self.mainLayout)


        self.grid = QtWidgets.QGridLayout()
        self.grid.setSpacing(10)
        self.mainLayout.addLayout(self.grid)


        row = 0
        self.chkActive = QtWidgets.QCheckBox()
        self.chkActive.setText("Active")
        self.grid.addWidget(self.chkActive, row, 1)

        row += 1
        self.grid.addWidget(QtWidgets.QLabel("Name"), row, 0, Qt.AlignRight)
        self.txtName = cwidgets.XLineEdit()
        self.grid.addWidget(self.txtName, row, 1, 1, 2)

        row += 1
        self.grid.addWidget(QtWidgets.QLabel("Description"), row, 0, Qt.AlignRight|Qt.AlignTop)
        self.txtDescription = QtWidgets.QTextEdit()
        self.txtDescription.setFixedHeight(100)
        self.grid.addWidget(self.txtDescription, row, 1, 1, 2)



        self.grid.setColumnStretch(0, 1)
        self.grid.setColumnStretch(1, 1)
        self.grid.setColumnStretch(2, 2)

        self.formBar = cwidgets.FormActionBar(self)
        self.mainLayout.addWidget(self.formBar)

        self.init_load()

    def init_load(self):
        self.serverConn.get(self, self.url)

    def on_server_reply(self, reply):

        if reply.busy:
            return

        print(reply.data)
        if reply.post:

            if reply.data.get("saved") == True:
                self.sigSaved.emit(reply.data['program'])
                self.accept()
            return

        self.formBar.set_reply(reply)

        rec = reply.data.get('program')

        self.chkActive.setChecked(rec['active'])
        self.txtDescription.setText(rec['description'])
        self.txtName.setText(rec['name'])
        #self.txtEmail.setText(rec['email'])

        self.formBar.set_meta("prog_id", rec)

        self.setDisabled(False)
        self.txtName.setFocus()



    def on_cancel(self):
        self.reject()

    def on_save(self):

        if self.txtName.len() < 4:
            self.txtName.setFocus()
            return


        rec = dict(active=self.chkActive.isChecked(),
                   description=self.txtDescription.toPlainText(),
                   name=self.txtName.s(),
                   prog_id= self.prog_id
                   )

        self.serverConn.post(self, self.url, data=rec)


    def on_delete(self):
        self.sigDeleted.emit(self.user_id)
        self.accept()

    def on_set_pass(self, state=False):
        self.txtPass.setEnabled(self.chkPass.isChecked())


# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico

import cwidgets

class MbConfigDialog(QtWidgets.QDialog):

    def __init__(self, parent, serverConn):
        QtWidgets.QDialog.__init__(self, parent)

        assert serverConn != None
        self.serverConn = serverConn

        self.setMinimumWidth(800)
        self.setMinimumHeight(800)


        self.setWindowTitle("mbconfig.cfg")
        self.setWindowIcon(Ico.icon(Ico.mbconfig))

        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        self.txt = QtWidgets.QTextEdit()
        self.mainLayout.addWidget(self.txt)

        self.fetch()

    def fetch(self):

        self.serverConn.get(self, "/modbus-config")


    def on_server_reply(self, reply):
        if reply.busy:
            return
        print(reply)
        self.txt.setPlainText(reply.data['mbconfig'])




import sys
from builtins import print
import os
import getpass
import socket
from pathlib import Path
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton, QPlainTextEdit, QDesktopWidget
from PyQt5.QtGui import QSyntaxHighlighter, QTextCharFormat, QColor, QFont, QTextCursor
from PyQt5.QtCore import Qt, pyqtSignal, QRegExp, QProcess, QThread


class PlainTextEdit(QPlainTextEdit):
    commandSignal = pyqtSignal(str)
    commandZPressed = pyqtSignal(str)

    def __init__(self, parent=None, movable=False):
        super().__init__(parent)

        self.name = "[" + str(getpass.getuser()) + "@" + str(socket.gethostname()) + "]" + "  ~" + str(
            os.getcwd()) + " >$ "
        self.appendPlainText(self.name)
        self.movable = movable
        self.parent = parent
        self.commands = []  # This is a list to track what commands the user has used so we could display them when
        # up arrow key is pressed
        self.tracker = 0
        self.setStyleSheet("QPlainTextEdit{background-color: #212121; color: white; padding: 8;}")
        self.font = QFont()
        self.font.setFamily("Iosevka")
        self.font.setPointSize(12)
        self.text = None
        self.setFont(self.font)
        self.document_file = self.document()
        self.previousCommandLength = 0
        self.document_file.setDocumentMargin(-1)

    def center(self):
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def mousePressEvent(self, event):
        if self.movable is True:
            self.parent.mousePressEvent(event)

    def mouseMoveEvent(self, event):
        if self.movable is True:
            self.parent.mouseMoveEvent(event)

    def textUnderCursor(self):
        textCursor = self.textCursor()
        textCursor.select(QTextCursor.WordUnderCursor)

        return textCursor.selectedText()

    def keyPressEvent(self, e):
        cursor = self.textCursor()

        if self.parent:

            if e.modifiers() == Qt.ControlModifier and e.key() == Qt.Key_A:
                return

            if e.modifiers() == Qt.ControlModifier and e.key() == Qt.Key_Z:
                self.commandZPressed.emit("True")
                return

            if e.key() == 16777220:  # This is the ENTER key
                text = self.textCursor().block().text()

                if text == self.name + text.replace(self.name, "") and text.replace(self.name,
                                                                                    "") != "":  # This is to prevent adding in commands that were not meant to be added in
                    self.commands.append(text.replace(self.name, ""))
                self.commandSignal.emit(text)
                self.appendPlainText(self.name)

                return

            if e.key() == Qt.Key_Up:
                try:
                    if self.tracker != 0:
                        cursor.select(QTextCursor.BlockUnderCursor)
                        cursor.removeSelectedText()
                        self.appendPlainText(self.name)

                    self.insertPlainText(self.commands[self.tracker])
                    self.tracker += 1

                except IndexError:
                    self.tracker = 0

                return

            if e.key() == Qt.Key_Down:
                try:
                    cursor.select(QTextCursor.BlockUnderCursor)
                    cursor.removeSelectedText()
                    self.appendPlainText(self.name)

                    self.insertPlainText(self.commands[self.tracker])
                    self.tracker -= 1

                except IndexError:
                    self.tracker = 0

            if e.key() == 16777219:
                if cursor.positionInBlock() <= len(self.name):
                    return

                else:
                    cursor.deleteChar()

            super().keyPressEvent(e)

        e.accept()


class Terminal(QWidget):
    errorSignal = pyqtSignal(str)
    outputSignal = pyqtSignal(str)

    def __init__(self, parent, movable=False):
        super().__init__()

        self.setWindowFlags(
            Qt.Widget |
            Qt.WindowCloseButtonHint |
            Qt.WindowStaysOnTopHint |
            Qt.FramelessWindowHint
        )
        self.movable = movable
        self.layout = QVBoxLayout()
        self.pressed = False
        self.process = QProcess()
        self.parent = parent
        self.name = None

        self.process.readyReadStandardError.connect(self.onReadyReadStandardError)
        self.process.readyReadStandardOutput.connect(self.onReadyReadStandardOutput)
        self.setLayout(self.layout)
        self.setStyleSheet("QWidget {background-color:invisible;}")

        # self.showMaximized() # comment this if you want to embed this widget

    def ispressed(self):
        return self.pressed

    def center(self):
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def add(self):
        self.added()
        self.button = QPushButton("Hide terminal")
        # self.button.setFont(QFont("Iosevka", 11))
        # self.button.setStyleSheet("""
        # height: 20;
        # background-color: #212121;
        #
        # """)
        self.button.setFixedWidth(120)
        self.editor = PlainTextEdit(self, self.movable)
        self.highlighter = name_highlighter(self.editor.document(), str(getpass.getuser()), str(socket.gethostname()),
                                            str(os.getcwd()))
        self.layout.addWidget(self.button)
        self.layout.addWidget(self.editor)
        self.editor.commandSignal.connect(self.handle)
        self.button.clicked.connect(self.remove)
        self.editor.commandZPressed.connect(self.handle)

    def added(self):
        self.pressed = True

    def remove(self):
        self.editor.deleteLater()
        self.button.deleteLater()
        self.parent.hideConsole()
        self.pressed = False

    def onReadyReadStandardError(self):
        self.error = self.process.readAllStandardError().data().decode()
        self.editor.appendPlainText(self.error.strip('\n'))
        self.errorSignal.emit(self.error)

    def onReadyReadStandardOutput(self):
        self.result = self.process.readAllStandardOutput().data().decode()
        self.editor.appendPlainText(self.result.strip('\n'))
        self.state = self.process.state()
        self.outputSignal.emit(self.result)

    def run(self, command):
        """Executes a system command."""
        if self.process.state() != 2:
            self.process.start(command)

    def handle(self, command):

        """Split a command into list so command echo hi would appear as ['echo', 'hi']"""
        real_command = command.replace(self.editor.name, "")

        if command == "True":
            if self.process.state() == 2:
                self.process.kill()
                self.editor.appendPlainText("Program execution killed, press enter")

        if real_command.startswith("python"):
            pass

        if real_command != "":
            command_list = real_command.split()
        else:
            command_list = None
        """Now we start implementing some commands"""
        if real_command == "clear":
            self.editor.clear()

        elif command_list is not None and command_list[0] == "echo":
            self.editor.appendPlainText(" ".join(command_list[1:]))

        elif real_command == "exit":
            self.remove()

        elif command_list is not None and command_list[0] == "cd" and len(command_list) > 1:
            try:
                os.chdir(" ".join(command_list[1:]))
                self.editor.name = "[" + str(getpass.getuser()) + "@" + str(socket.gethostname()) + "]" + "  ~" + str(
                    os.getcwd()) + " >$ "
                if self.highlighter:
                    del self.highlighter
                self.highlighter = name_highlighter(self.editor.document(), str(getpass.getuser()),
                                                    str(socket.gethostname()), str(os.getcwd()))

            except FileNotFoundError as E:
                self.editor.appendPlainText(str(E))

        elif command_list is not None and len(command_list) == 1 and command_list[0] == "cd":
            os.chdir(str(Path.home()))
            self.editor.name = "[" + str(getpass.getuser()) + "@" + str(socket.gethostname()) + "]" + "  ~" + str(
                os.getcwd()) + " >$ "

        elif self.process.state() == 2:
            self.process.write(real_command.encode())
            self.process.closeWriteChannel()

        elif command == self.editor.name + real_command:
            self.run(real_command)

        else:
            pass
    # When the user does a command like ls and then presses enter then it wont read the line where the cursor is on as a command


class name_highlighter(QSyntaxHighlighter):

    def __init__(self, parent=None, user_name=None, host_name=None, cwd=None):
        super().__init__(parent)
        self.highlightingRules = []
        self.name = user_name
        self.name2 = host_name
        self.cwd = cwd
        most_used = ["cd", "clear", "history", "ls", "man", "pwd", "what", "type",
                     "strace", "ltrace", "gdb", "cat", "chmod", "cp", "chown", "find", "grep", "locate", "mkdir",
                     "rmdir", "rm", "mv", "vim", "nano", "rename",
                     "touch", "wget", "zip", "tar", "gzip", "apt", "bg", "fg", "df", "free", "ip", "jobs", "kill",
                     "killall", "mount", "umount", "ps", "sudo", "echo",
                     "top", "uname", "whereis", "uptime", "whereis", "whoami", "exit"
                     ]  # most used linux commands, so we will highlight them!
        self.regex = {
            "class": "\\bclass\\b",
            "function": "[A-Za-z0-9_]+(?=\\()",
            "magic": "\\__[^']*\\__",
            "decorator": "@[^\n]*",
            "singleLineComment": "#[^\n]*",
            "quotation": "\"[^\"]*\"",
            "quotation2": "'[^\']*\'",
            "multiLineComment": "[-+]?[0-9]+",
            "int": "[-+]?[0-9]+",
        }
        """compgen -c returns all commands that you can run"""

        for f in most_used:
            nameFormat = QTextCharFormat()
            nameFormat.setForeground(QColor("#00ff00"))
            nameFormat.setFontItalic(True)
            self.highlightingRules.append((QRegExp("\\b" + f + "\\b"), nameFormat))

        hostnameFormat = QTextCharFormat()
        hostnameFormat.setForeground(QColor("#12c2e9"))
        self.highlightingRules.append((QRegExp(self.name), hostnameFormat))
        self.highlightingRules.append((QRegExp(self.name2), hostnameFormat))

        otherFormat = QTextCharFormat()
        otherFormat.setForeground(QColor("#f7797d"))
        self.highlightingRules.append((QRegExp("~\/[^\s]*"), otherFormat))

        quotation1Format = QTextCharFormat()
        quotation1Format.setForeground(QColor("#96c93d"))
        self.highlightingRules.append((QRegExp("\"[^\"]*\""), quotation1Format))

        quotation2Format = QTextCharFormat()
        quotation2Format.setForeground(QColor("#96c93d"))
        self.highlightingRules.append((QRegExp("'[^\']*\'"), quotation2Format))

        integerFormat = QTextCharFormat()
        integerFormat.setForeground(QColor("#cc5333"))
        integerFormat.setFontItalic(True)
        self.highlightingRules.append((QRegExp("\\b[-+]?[0-9]+\\b"), integerFormat))

    def highlightBlock(self, text):

        for pattern, format in self.highlightingRules:
            expression = QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format)
                index = expression.indexIn(text, index + length)


class PythonThread(QThread):

    def __init__(self):
        super().__init__()








from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico

from programs import programs_model, program_dialog


class ProgramsGrid(QtWidgets.QWidget):



    def __init__(self, parent, serverConn=None):
        QtWidgets.QWidget.__init__(self, parent)

        self.serverConn = serverConn
        assert self.serverConn != None

        self.model = programs_model.ProgramsModel(self, serverConn=self.serverConn)


        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)



        #===================
        # Toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.mainLayout.addWidget(self.toolbar)

        #  Actions
        grp = cwidgets.ToolBarGroup(self, "Program Actions")
        self.toolbar.addWidget(grp)

        self.actionAdd = cwidgets.XToolButton(text="Add", ico=Ico.add, callback=self.on_add)
        grp.addWidget(self.actionAdd)

        self.actionEdit = cwidgets.XToolButton(text="Edit", ico=Ico.edit, callback=self.on_edit)
        grp.addWidget(self.actionEdit)
        self.actionEdit.setDisabled(True)




        self.tree = QtWidgets.QTreeView()
        self.mainLayout.addWidget(self.tree)

        self.tree.setModel(self.model)
        self.tree.doubleClicked.connect(self.on_tree_double)
        self.tree.selectionModel().selectionChanged.connect(self.on_tree_selection)



        self.statusBar = cwidgets.StatusBar(self)
        self.mainLayout.addWidget(self.statusBar)


        self.model.sigLoaded.connect(self.on_loaded)

    def init_load(self):
        self.fetch()

    def on_refresh(self):
        self.fetch()

    def fetch(self):
        self.model.fetch()



    def on_add(self):

        self.show_edit_dialog("new")

    def on_edit(self):
        sm = self.tree.selectionModel()
        if not sm.hasSelection():
            return
        rec = self.model.recs[sm.selectedIndexes()[0].row()]
        self.show_edit_dialog(rec['prog_id'])

    def show_edit_dialog(self, prog_id=None):

        dial = program_dialog.ProgramDialog(self, prog_id=prog_id, serverConn=self.serverConn)
        dial.sigSaved.connect(self.on_rec_saved)
        dial.exec_()

    def on_rec_saved(self, rec):
        self.model.upsert(rec)



    def on_tree_double(self):
        self.on_edit()

    def on_tree_selection(self, sel=None, desel=None):
        ena = self.tree.selectionModel().hasSelection()
        self.actionEdit.setEnabled(ena)

    def on_loaded(self):
        C = programs_model.C
        for c in [C.active, C.name]:
            self.tree.resizeColumnToContents(c)
        self.tree.setColumnWidth(C.description, 300)







from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico

from slave_devices import slave_devices_model, slave_dialog


class SlaveDevicesGrid(QtWidgets.QWidget):



    def __init__(self, parent, serverConn=None):
        QtWidgets.QWidget.__init__(self, parent)

        self.serverConn = serverConn
        assert self.serverConn != None

        self.model = slave_devices_model.SlaveDevicesModel(self, serverConn=self.serverConn)

        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)



        #===================
        # Toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.mainLayout.addWidget(self.toolbar)

        # Actions
        grp = cwidgets.ToolBarGroup(self, "Slave Actions")
        self.toolbar.addWidget(grp)

        self.actionAdd = cwidgets.XToolButton(text="Add", ico=Ico.add, callback=self.on_add)
        grp.addWidget(self.actionAdd)

        self.actionEdit = cwidgets.XToolButton(text="Edit", ico=Ico.edit, callback=self.on_edit)
        grp.addWidget(self.actionEdit)
        self.actionEdit.setDisabled(True)




        self.tree = QtWidgets.QTreeView()
        self.mainLayout.addWidget(self.tree)

        self.tree.setModel(self.model)
        self.tree.doubleClicked.connect(self.on_tree_double)
        self.tree.selectionModel().selectionChanged.connect(self.on_tree_selection)



        self.statusBar = cwidgets.StatusBar(self)
        self.mainLayout.addWidget(self.statusBar)



    def init_load(self):
        self.fetch()

    def on_refresh(self):
        self.fetch()

    def fetch(self):
        self.model.fetch()



    def on_add(self):

        self.show_edit_dialog("new")

    def on_edit(self):
        sm = self.tree.selectionModel()
        if not sm.hasSelection():
            return
        rec = self.model.recs[sm.selectedIndexes()[0].row()]
        self.show_edit_dialog(rec['dev_id'])

    def show_edit_dialog(self, dev_id=None):

        dial = slave_dialog.SlaveDialog(self, dev_id=dev_id, serverConn=self.serverConn)
        dial.sigSaved.connect(self.on_rec_saved)
        dial.exec_()

    def on_rec_saved(self, rec):
        self.model.upsert(rec)



    def on_tree_double(self):
        self.on_edit()

    def on_tree_selection(self, sel=None, desel=None):
        ena = self.tree.selectionModel().hasSelection()
        self.actionEdit.setEnabled(ena)
        #self.actionDelete.setEnabled(ena)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Peter Morgan <pete@daffodil.uk.com>
@copyright: Peter Morgan

"""

import argparse

import desktop
import desktop.main_window

parser = argparse.ArgumentParser(description="Launch desktop application")
parser.add_argument("--pedro", help="Pedro's Developer mode", action="store_true")

if __name__ == '__main__':


    args = parser.parse_args()

    desktop.start_app(desktop.main_window.MainWindow, args)






from Qt import Qt, QtGui, QtCore, pyqtSignal


class C:
    node = 0

    dev_id = 1
    dev_name = 2
    dev_type = 3
    conn_type = 4
    slave_id = 5

    com_port = 6
    baud_rate = 7
    data_bits = 8
    stop_bits = 9

    ip_address = 10
    ip_port = 11

    di_start = 12
    di_size = 13

    coil_start = 14
    coil_size = 15

    ir_start = 16
    ir_size = 17

    hr_read_start = 18
    hr_read_size = 19

    hr_write_start = 20
    hr_write_size = 21

    active = 22

    _count = 23


class SlaveDevicesModel(QtCore.QAbstractTableModel):

    sigLoaded = pyqtSignal()

    def data(self, midx, role):

        rec = self.recs[midx.row()]
        cidx = midx.column()

        if role == Qt.DisplayRole:
            if cidx == C.active:
                return "Yes" if rec['active'] else "-"

            elif cidx == C.dev_id:
                return rec['dev_id']

            elif cidx == C.dev_name:
                return rec['dev_name']

            elif cidx == C.dev_type:
                return rec['dev_type']

            elif cidx == C.conn_type:
                return rec['conn_type']

            elif cidx == C.slave_id:
                return rec['slave_id']

            elif cidx == C.com_port:
                return rec['com_port']
            elif cidx == C.baud_rate:
                return rec['baud_rate']
            elif cidx == C.data_bits:
                return rec['data_bits']
            elif cidx == C.stop_bits:
                return rec['stop_bits']

            elif cidx == C.ip_address:
                return rec['ip_address']
            elif cidx == C.ip_port:
                return rec['ip_port']

            elif cidx == C.di_start:
                return rec['di_start']
            elif cidx == C.di_size:
                return rec['di_size']


            elif cidx == C.ir_start:
                return rec['ir_start']
            elif cidx == C.ir_size:
                return rec['ir_size']

            elif cidx == C.coil_start:
                return rec['coil_start']
            elif cidx == C.coil_size:
                return rec['coil_size']

            elif cidx == C.hr_read_start:
                return rec['hr_read_start']
            elif cidx == C.hr_read_size:
                return rec['hr_read_size']

            elif cidx == C.hr_write_start:
                return rec['hr_write_start']
            elif cidx == C.hr_write_size:
                return rec['hr_write_size']

            elif cidx == C.dev_id:
                return  rec['dev_id']

        if role == Qt.BackgroundRole:
            if cidx == C.active:
                return QtGui.QColor("#D8FFB2") if rec['active'] else QtGui.QColor("white")

    def __init__(self, parent=None, serverConn=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        self.serverConn = serverConn
        self.recs = []


    def columnCount(self, pidx=None):
        return C._count

    def rowCount(self, pidx=None):
        return len(self.recs)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["", "dev_id", "Name", "Type", "Conn Type", "Slave ID",
                    "Com Port", "Baud Rate", "Data Bits", "Stop Bits",
                    "IP Address", "IP Port",
                    "DI Start", "DI Size",
                    "Coil Start", "Coil Size",
                    "IR Start", "IR Size",
                    "HR Read Start", "HR Read Size",
                    "HR Write Start", "HR Write Size",
                    "Active"
                    ][idx]

    def fetch(self):
         self.serverConn.get(self, "/slave-devices")

    def on_server_reply(self, reply):
        if reply.busy:
            return

        if reply.data == None:
            return

        self.recs = reply.data['slaves']

        self.modelReset.emit()
        self.sigLoaded.emit()

    def upsert(self, rec):
        for idx, r in enumerate(self.recs):
            if r['dev_id'] == rec['dev_id']:
                self.recs[idx] = rec
                self.layoutChanged.emit()
                return
        self.recs.append(rec)
        self.layoutChanged.emit()



# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico
import G


from server import server_dialog, servers_model



class ServersDialog(QtWidgets.QDialog):



    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)

        self.model = servers_model.ServersModel()


        self.setWindowIcon(Ico.icon(Ico.servers))
        self.setWindowTitle("Servers")

        self.setMinimumWidth(500)
        self.setMinimumHeight(500)

        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)


        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.mainLayout.addWidget(self.toolbar)

        # Creds

        self.actionAdd = cwidgets.XToolButton(text="Add", ico=Ico.add, callback=self.on_add)
        self.toolbar.addWidget(self.actionAdd)

        self.actionEdit = cwidgets.XToolButton(text="Edit", ico=Ico.edit, callback=self.on_edit)
        self.toolbar.addWidget(self.actionEdit)

        self.actionDelete = cwidgets.XToolButton(text="Delete", ico=Ico.delete, callback=self.on_delete)
        self.toolbar.addWidget(self.actionDelete)

        self.tree = QtWidgets.QTreeView()
        self.mainLayout.addWidget(self.tree)

        self.tree.setModel(self.model)
        self.tree.doubleClicked.connect(self.on_tree_double)
        self.tree.selectionModel().selectionChanged.connect(self.on_tree_selection)

        self.on_tree_selection()
        self.load()



    def load(self):
       self.model.load()

    def on_tree_double(self):
        self.on_edit()

    def on_tree_selection(self, sel=None, desel=None):
        ena = self.tree.selectionModel().hasSelection()
        self.actionEdit.setEnabled(ena)
        self.actionDelete.setEnabled(ena)

    def on_add(self):
        self.show_edit_dialog()

    def on_edit(self):
        sm = self.tree.selectionModel()
        if not sm.hasSelection():
            return
        rec = self.model.recs[sm.selectedIndexes()[0].row()]
        self.show_edit_dialog(rec)

    def show_edit_dialog(self, srv=None):

        dial = server_dialog.ServerDialog(self, server=srv)
        if dial.exec_():
            self.load()

    def on_delete(self):
        sm = self.tree.selectionModel()
        if not sm.hasSelection():
            return

        resp = QtWidgets.QMessageBox.question(self, "Delete ?", "delete server")
        if resp == QtWidgets.QMessageBox.Yes:
            rec = self.model.recs[sm.selectedIndexes()[0].row()]#
            G.settings.remove_server(rec)
            self.load()





# -*- coding: utf-8 -*-

from PyQt5 import QtCore
from PyQt5.QtCore import Qt

from PyQt5 import QtGui
from PyQt5 import QtWidgets

from PyQt5 import QtNetwork

#from PyQt5 import QtWebSockets
#from PyQt5 import QtWebKit
#from PyQt5 import QtWebKitWidgets
#from PyQt5 import QtSql


from PyQt5.QtCore import pyqtSignal




# -*- coding: utf-8 -*-

import paramiko
from Qt import QtCore, QtWidgets, QtNetwork, Qt, pyqtSignal

from img import Ico

import cwidgets


class RuntimeCommanderWidget(QtWidgets.QWidget):

    def __init__(self, parent, serverConn):
        QtWidgets.QWidget.__init__(self, parent)

        assert serverConn != None
        self.serverConn = serverConn

        self.serverConn.sigRuntimeMessage.connect(self.on_runtime_message)


        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)

        # self.toolbar = QtWidgets.QToolBar()
        # self.mainLayout.addWidget(self.toolbar)



        # ===============
        midLay = cwidgets.hlayout()
        self.mainLayout.addLayout(midLay)
        #grp = cwidgets.ToolBarGroup(title="Commands")
        #self.toolbar.addWidget(grp)

        self.txt = QtWidgets.QTextEdit()
        midLay.addWidget(self.txt)


        rightLay = cwidgets.vlayout(spacing=5)
        midLay.addLayout(rightLay)

        grp = cwidgets.ToolBarGroup(title="RunTime Connect")
        rightLay.addWidget(grp)
        self.buttConnect = cwidgets.XToolButton(ico=Ico.start, text="Connect", callback=self.on_connect)
        grp.addWidget(self.buttConnect)

        self.buttConnect = cwidgets.XToolButton(ico=Ico.start, text="SSH", callback=self.on_ssh)
        grp.addWidget(self.buttConnect)

        self.lblStatus = cwidgets.XLabel()
        grp.addWidget(self.lblStatus)



        rightLay.addSpacing(10)
        lbl = cwidgets.XLabel(text="Commands")
        rightLay.addWidget(lbl)

        self.buttGroup = QtWidgets.QButtonGroup(self)
        self.buttGroup.setExclusive(False)
        self.buttGroup.buttonClicked.connect(self.on_exec_button)
        for cmd in ["runtime_logs()", "exec_time()",
                    "start_modbus()", "stop_modbus()",
                    "start_dnp3()", "stop_dnp3()",
                    "start_enip()", "stop_enip()",
                    "start_pstorage()", "stop_pstorage()",
                    "quit()"
                    ]:
            butt = cwidgets.XToolButton(text=cmd, autoRaise=False)
            rightLay.addWidget(butt)
            self.buttGroup.addButton(butt)
            butt.setFixedWidth(200)
            butt.setDisabled(True)

        rightLay.addStretch(10)

        self.show_status(False)


    def init_load(self):
        pass


    def on_connect(self):
        self.serverConn.connect_runtime()

    def on_runtime_message(self, mess):
        print(mess)
        self.show_status(mess.connected)
        if mess.message:
            self.txt.setPlainText(mess.message)
        for b in self.buttGroup.buttons():
            b.setEnabled(mess.connected)

    def on_exec_button(self, butt):
        cmd = butt.text()
        self.serverConn.send_runtime(cmd)


    def show_status(self, connected):
        self.lblStatus.setText("Connected" if connected else "Disconnected")
        sty = "background-color: %s" % ("#EDFFDA" if connected else "#FFDFDA")
        self.lblStatus.setStyleSheet(sty)

    def on_ssh(self):
        client = paramiko.SSHClient()
        client.load_system_host_keys()
        client.set_missing_host_key_policy(paramiko.WarningPolicy)

        print("on_ssh")
        client.connect("192.168.1.222", port=22, username="pi", password="raspberry")
        print("connect")
        stdin, stdout, stderr = client.exec_command("ls -al")
        #print( stdout.read(), )
        s = str(stdout.read(), encoding="ascii")
        print(s)
        self.txt.setText(s)


import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

with open("version.txt", "r") as fh:
    version = fh.read()

setuptools.setup(
    name="openplc-desktop", # Replace with your own username
    version=version,
    author="Pedro Morgan",
    author_email="openplc@daffodil.uk.com",
    description="OpenPLC Desktop",
    long_description=long_description,
    #long_description_content_type="text/markdown",
    url="https://openplcproject.gitlab.io/openplc_desktop/",
    project_urls={
        "Bug Tracker": "https://gitlab.com/openplcproject/openplc_desktop/issues",
        "Documentation": "https://openplcproject.gitlab.io/openplc_desktop/",
        "Source Code": "https://gitlab.com/openplcproject/openplc_desktop",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    scripts=['openplc-desktop.py'],
)




import datetime


from Qt import QtCore, QtGui, QtWidgets, Qt, pyqtSignal

import cwidgets
from img import Ico




class WebSocketPanel(QtWidgets.QWidget):



    def __init__(self, parent=None, serverConn=None):
        QtWidgets.QWidget.__init__(self, parent)


        self.serverConn = serverConn


        self.mainLayout = cwidgets.vlayout()
        self.setLayout(self.mainLayout)


        self.toolbar = QtWidgets.QToolBar()
        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.mainLayout.addWidget(self.toolbar)

        # # Actions
        self.tbActions = cwidgets.ToolBarGroup(self, title="Actions")
        self.toolbar.addWidget(self.tbActions)
        #
        # self.lblServerName = cwidgets.XLabel()
        # self.lblServerName.setFixedWidth(160)
        # self.tbServer.addWidget(self.lblServerName)
        #
        # self.lblServerStatus = cwidgets.XLabel()
        # self.lblServerStatus.setFixedWidth(160)
        # self.tbServer.addWidget(self.lblServerStatus)
        #
        self.buttClear = cwidgets.XToolButton(self, ico=Ico.clear, text="Clear", callback=self.on_clear)
        self.tbActions.addWidget(self.buttClear)
        #
        # self.buttWs = cwidgets.XToolButton(self, ico=Ico.save, both=False, callback=self.on_ws)
        # self.tbServer.addWidget(self.buttWs)

        self.model = WebSocketModel(serverConn=serverConn)

        #self.proxy = QtCore.QSortFilterProxyModel()
        #self.proxy.setSourceModel(self.model)


        ## Midx widgets
        self.tree = QtWidgets.QTreeView()
        self.mainLayout.addWidget(self.tree)

        self.tree.setModel(self.model)

        for c in [C.ts, C.ns, C.mtype]:
            self.tree.setColumnHidden(c, True)

        #self.tree.sortByColumn(C.ts, Qt.DescendingOrder)




    def init_load(self):

        # self.fetch()
        #self.on_login()
        pass

    def on_clear(self):
        self.model.clear()

class C:
    ts = 0
    ns = 1
    mtype = 2
    data = 3

    _count = 4


class WebSocketModel(QtCore.QAbstractTableModel):

    sigLoaded = pyqtSignal()

    def data(self, midx, role):

        rec = self.recs[midx.row()]
        cidx = midx.column()

        if role == Qt.DisplayRole:
            if cidx == C.ns:
                return rec['ns']

            elif cidx == C.mtype:
                return rec['mtype']

            elif cidx == C.data:
                return rec['data']

            elif cidx == C.ts:
                return rec['ts']

    def clear(self):
        del self.recs[:]
        self.layoutChanged.emit()


    def __init__(self, parent=None, serverConn=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        assert serverConn != None
        self.serverConn = serverConn
        self.recs = []

        self.serverConn.sigWsMessage.connect(self.on_ws_message)

    def on_ws_message(self, recs):
        for r in recs:
            self.recs.append(r)
        self.layoutChanged.emit()


    def columnCount(self, pidx=None):
        return C._count

    def rowCount(self, pidx=None):
        return len(self.recs)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["ts", "Ns", "Type", "Data"][idx]




# -*- coding: utf-8 -*-

from Qt import QtCore, QtWidgets, Qt, pyqtSignal

from img import Ico
import G

class C:
    active = 0
    name = 1
    date_upload = 2
    description = 3
    prog_id = 4

    _count = 5


class ProgramsModel(QtCore.QAbstractTableModel):

    sigLoaded = pyqtSignal()

    def data(self, midx, role):

        rec = self.recs[midx.row()]
        cidx = midx.column()

        if role == Qt.DisplayRole:

            if cidx == C.active:
                return "Yes" if rec["active"] else "-"

            if cidx == C.name:
                return rec["name"]

            elif cidx == C.date_upload:
                return rec['date_upload']

            elif cidx == C.description:
                return rec['description']

            elif cidx == C.prog_id:
                return rec['prog_id']

        if role == Qt.DecorationRole and  cidx == C.name:
            return Ico.icon(Ico.program)



    def __init__(self, parent=None, serverConn=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        assert serverConn != None
        self.serverConn = serverConn

        self.recs = []


    def columnCount(self, pidx=None):
        return C._count

    def rowCount(self, pidx=None):
        return len(self.recs)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["Active", "Name", "Upload", "Description", "ProgID"][idx]


    def fetch(self):
        self.serverConn.get(self, "/programs")

    def on_server_reply(self, reply):

        if reply.busy:
            return


        self.recs = reply.data['programs']
        self.modelReset.emit()
        self.sigLoaded.emit()

    def upsert(self, rec):
        for idx, r in enumerate(self.recs):
            if r['prog_id'] == rec['prog_id']:
                self.recs[idx] = rec
                self.layoutChanged.emit()
                return
        self.recs.append(rec)
        self.layoutChanged.emit()





import os
import sys

HERE_PATH =  os.path.abspath(os.path.dirname(__file__))
PROJECT_ROOT = os.path.abspath(os.path.join(HERE_PATH, ".."))

if not HERE_PATH in sys.path:
    sys.path.insert(0, HERE_PATH)


from Qt import QtGui, QtWidgets


class EXIT_CODE:
    restart = 10
    upgrade = 11
    crash = 99
    no_db = 98
    quit = 0


def show_splash():
    # splashImage = QtGui.QPixmap( "../images/corp/splash.png" )
    splashScreen = QtWidgets.QSplashScreen()
    splashScreen.showMessage("  Loading . . . ")
    splashScreen.show()
    return splashScreen


def start_app(appWidget, args, splash=True):
    splashScreen = None

    app = QtWidgets.QApplication(sys.argv)
    # if len(app.arguments()) == 1:
    #	print("FATAL: no database")
    #	sys.exit(EXIT_CODE.no_db)

    ## Main Window
    window = appWidget(args=args)  # launcher=launcher, debug=debug, dbserver=dbserver, full_screen=full_screen)

    # if app_globals.settings.is_dev() == False and splash== True:
    #	splashScreen.finish( window )

    window.show()

    exit_code = app.exec_()
    sys.exit(exit_code)





from Qt import Qt, QtGui, QtCore, pyqtSignal


class C:
    node = 0

    user_id = 1
    login = 2
    pass_hash = 3
    name = 4
    email = 5

    active = 6
    date_created = 7
    date_updated = 8

    _cols_count = 9


class UsersModel(QtCore.QAbstractTableModel):

    sigLoaded = pyqtSignal()

    def data(self, midx, role):

        rec = self.recs[midx.row()]
        cidx = midx.column()

        if role == Qt.DisplayRole:
            if cidx == C.active:
                return "Yes" if rec['active'] else "-"

            elif cidx == C.user_id:
                return rec['user_id']

            elif cidx == C.login:
                return rec['login']

            elif cidx == C.pass_hash:
                return rec['password_hash']

            elif cidx == C.name:
                return rec['name']

            elif cidx == C.date_created:
                return rec['date_created']

            elif cidx == C.date_updated:
                return rec['date_updated']


        if role == Qt.BackgroundRole:
            if cidx == C.active:
                return QtGui.QColor("#D8FFB2") if rec['active'] else QtGui.QColor("white")

    def __init__(self, parent=None, serverConn=None):
        super(QtCore.QAbstractTableModel, self).__init__()

        self.serverConn = serverConn
        self.recs = []


    def columnCount(self, pidx=None):
        return C._cols_count

    def rowCount(self, pidx=None):
        return len(self.recs)

    def headerData(self, idx, orientation, role):
        if role == Qt.DisplayRole:
            return ["", "UserID", "Syslogin", "Pass", "Full Name", "Email",
                    "Active", "Created", "Updated"][idx]

    def fetch(self):
         self.serverConn.get(self, "/users")

    def on_server_reply(self, reply):
        if reply.busy:
            return

        #print(reply)
        if reply.data == None:
            return

        self.recs = reply.data['users']

        self.modelReset.emit()
        self.sigLoaded.emit()




# -*- coding: utf-8 -*-

import os
from Qt import  QtWidgets, QtGui
import G


# see https://github.com/spyder-ide/qtawesome

import qtawesome as qta

class Ico():

    caret_down = "fa5s.angle-down"

    add = "fa5s.plus"
    edit = "fa5.edit"
    delete = "fa5.trash-alt"

    cancel = "ei.remove"
    save = "ei.ok-circle"

    settings = "fa5s.cog"
    refresh = "ei.refresh"
    login = "ei.lock"

    start = "fa5s.play"
    stop = "fa5s.stop"

    up = "ei.arrow-up"
    down = "ei.arrow-down"

    quit = "ei.eject"
    clear = "ei.remove-circle"

    filter_on = "fa5s.server"
    filter_off = "fa5s.server"


    server = "fa5s.server"
    servers = "ei.th-list"

    slaves = "fa5s.sitemap"
    slave = "fa5s.microchip"
    mbconfig = "fa5s.cogs"

    commander = "fa5s.solar-panel"
    terminal = "fa5s.desktop"
    return_key = "ei.return-key"

    programs = "fa5s.window-restore"
    program = "fa5s.file-code"

    users = "fa5s.user-friends"
    user = "fa5s.user"

    @staticmethod
    def icon(name):
        return qta.icon(name)



    @staticmethod
    def logo():
        pth = os.path.join(G.ROOT_PATH, "static", "images", "favicon.png")
        return QtGui.QIcon(pth)

    @staticmethod
    def from_path(pth):
          return QtGui.QIcon(pth)